package application;

/*
 *  Copyright (C) 2018 Juergen Platzer 
 * 
 *  This file is part of JInfoViz.
 *  
 *  JInfoViz is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  JInfoViz is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *  
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with JInfoViz.  If not, see <http://www.gnu.org/licenses/>.
 */


import view.*;
import data.*;
import listeners.*;



import javax.swing.JFrame;
import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;
import javax.swing.*;
import javax.swing.event.*;
import java.io.File;
import javax.imageio.ImageIO;
import java.util.Vector;

import gui.FileDialogHelpers;

/**
 * This class creates and handles the functionality the interface of the program.
 * It also represents the window in which the parallel coordinates view is shown and all functionalities
 * are called via the menus.
 *
 * @author    J&uuml;rgen Platzer
 */
//class creates and handles the functionality the interface of the application
public class Mainframe extends JFrame implements WindowListener, ActionListener, KeyListener, IDataSubsetListener{

  /*** member variables ***/

  private Data m_pDataset;         //data set
  private Data m_pWholeDataset;    //data set that is a backup, if selected data points are interactively chosen for further work
  private ParCoordView m_pParCoordView;  //object that draws the parallel coordinates
  private SelectionList         m_pSelectionList;  //selection list for DOI calculations


  /** frames of the menus **/
  private ColoringFrame         m_pColoringFrame;          //jframe that manages the coloring options
  private ClusteringFrame       m_pClusteringFrame;        //jframe that holds the initial settings for clustering
  private AxisManagementFrame   m_pAxisManagementFrame;           //jframe that steers the axis of the parallel coordinates
  private ImageExportFrame      m_pImageExportFrame;       //jframe that handles the export of the images
  private PCAFrame              m_pPCAFrame;               //jframe that handles the principal component analysis
  private DistanceFrame         m_pDistanceFrame;          //jframe that handles the calculation of robust and mahalanobis distance
  private ScatterplotFrame      m_pScatterplotFrame;       //jframe that handles the 2d scatterplot
  private OutlierDetectionFrame m_pOutlierDetectionFrame;  //jframe that handles outlier detection


  /////////////////////////////////
  // CONSTRUCTOR                 //
  /////////////////////////////////

  /**
   * Constructor that creates the main window and objects of all other windows and views.
   * The needed listeners are registered.
   *
   */

  public Mainframe() {

    //1. set Window-Properties
    setResizable(true);
    enableEvents(AWTEvent.WINDOW_EVENT_MASK);
    setSize(new Dimension(805, 700));
    setTitle("Parallel Coordinates");

    //2. add Listener functionality
    addWindowListener(this);
    addKeyListener(this);


    //3. set Containers-Properties (layout)
    Container pContainer = this.getContentPane();
    pContainer.setLayout(new BorderLayout());

    //4. create menu
    //4.1. file menu
    JMenu pFileMenu = new JMenu("File");

    JMenuItem pFileMenuOpen = new JMenuItem("Open");
    pFileMenuOpen.addActionListener(this);
    pFileMenu.add(pFileMenuOpen);

    JMenuItem pFileMenuSave = new JMenuItem("Save selected items");
    pFileMenuSave.addActionListener(this);
    pFileMenu.add(pFileMenuSave);


    JMenuItem pFileMenuExpImg = new JMenuItem("Export image");
    pFileMenuExpImg.addActionListener(this);
    pFileMenu.add(pFileMenuExpImg);

    //4.2. application menu
    JMenu pApplicationMenu = new JMenu("Applications");

    JMenuItem pApplicationMenuScatterplot = new JMenuItem("2D Scatterplot");
    pApplicationMenuScatterplot.addActionListener(this);
    pApplicationMenu.add(pApplicationMenuScatterplot);

    JMenuItem pApplicationMenuOutlier = new JMenuItem("Outlier detection");
    pApplicationMenuOutlier.addActionListener(this);
    pApplicationMenu.add(pApplicationMenuOutlier);


    //4.3. view menu
    JMenu pViewMenu = new JMenu("View");

    JMenuItem pViewMenuColoring = new JMenuItem("Coloring");
    pViewMenuColoring.addActionListener(this);
    pViewMenu.add(pViewMenuColoring);

    JMenuItem pViewMenuAxisMan = new JMenuItem("Axis management");
    pViewMenuAxisMan.addActionListener(this);
    pViewMenu.add(pViewMenuAxisMan);



    //4.4. tools menu
    JMenu pToolsMenu = new JMenu("Tools");

    JMenuItem pToolsMenuClust = new JMenuItem("Clustering");
    pToolsMenuClust.addActionListener(this);
    pToolsMenu.add(pToolsMenuClust);

    JMenuItem pToolsMenuPCA = new JMenuItem("Principal Component Analysis");
    pToolsMenuPCA.addActionListener(this);
    pToolsMenu.add(pToolsMenuPCA);


    JMenuItem mDistances = new JMenuItem("Distances");
    mDistances.addActionListener(this);
    pToolsMenu.add(mDistances);

    //4.5. create the menubar
    JMenuBar pMenuBar = new JMenuBar();
    pMenuBar.add(pFileMenu);
    pMenuBar.add(pApplicationMenu);
    pMenuBar.add(pViewMenu);
    pMenuBar.add(pToolsMenu);
    setJMenuBar(pMenuBar);



    //5. set the member variables
    m_pParCoordView = null;
    m_pWholeDataset = null;

    //6. load the default data
    File pFile = new File("data/iris.csv");
    setNewData(CSVIO.readCSVFile(pFile), null);




  }

  ////////////////////////////////////////
  //ACTION LISTENER INTERFACE           //
  ////////////////////////////////////////

  /**
   * Handles the action that is initiated by choosing a MenuItem.
   *
   * @param e ActionEvent that identifies the MenuItem from which the event is triggered.
   */
  public void actionPerformed(ActionEvent e) {
    //System.out.println(e.getActionCommand());

    //1. 2d Scatterplot menu item
    if (e.getActionCommand().equals("2D Scatterplot")) {
      m_pScatterplotFrame.setVisible(true);
      return;
    }


    //2. Outlier detection menu item
    if (e.getActionCommand().equals("Outlier detection")) {
      showNotAvailableMessage();
      //TODO: bring this back into the program with a new math library
      /*
      RobustDistanceFrame pRobustDistanceFrame = new RobustDistanceFrame(m_pDataset, m_pOutlierDetectionFrame);
      pRobustDistanceFrame.setVisible(true);
      */
      return;
    }

    //3. coloring menu item
    if (e.getActionCommand().equals("Coloring")) {
      m_pColoringFrame.setVisible(true);
      return;
    }

    //4. axis management menu item
    if (e.getActionCommand().equals("Axis management")) {
      m_pAxisManagementFrame.setVisible(true);
      return;
    }


    //5. clustering menu item
    if (e.getActionCommand().equals("Clustering")) {
      m_pClusteringFrame.setVisible(true);
      return;
    }


    //6. PCA menu item
    if (e.getActionCommand().equals("Principal Component Analysis")) {
      showNotAvailableMessage();
      //TODO: bring this back into the program with a new math library
      /*
      m_pPCAFrame.setVisible(true);
      */
      return;
    }

    //7. Distances menu item
    if (e.getActionCommand().equals("Distances")) {
      showNotAvailableMessage();
      //TODO: bring this back into the program with a new math library
      /*
      m_pDistanceFrame.setVisible(true);
      */
      return;
    }



    //8. file menu items - open
    if (e.getActionCommand().equals("Open")) {

      //calculate the current directory of our programm
      //String mainFrameDir = ClassLoader.getSystemResource("MainFrame.class").getPath();
      //String basepath = mainFrameDir.substring(0, mainFrameDir.length() - "MainFrame.class".length());

      //get the file chooser and open the file dialog
      JFileChooser pFileChooser = FileDialogHelpers.getDataFileChooser();
      File pFile = FileDialogHelpers.showOpenFileDialog(pFileChooser, this);
      
      

      //if a file was chosen, read it
      if(pFile != null) {
        setNewData(CSVIO.readCSVFile(pFile), null);
        m_pWholeDataset = null;
      }

      return;
    }

    //9. file menu items - save selected items
    if (e.getActionCommand().equals("Save selected items")) {

      //calculate the current directory of our programm
      //String mainFrameDir = ClassLoader.getSystemResource("MainFrame.class").getPath();
      //String basepath = mainFrameDir.substring(0, mainFrameDir.length() - "MainFrame.class".length());

      //create and modify FileChooser
      JFileChooser pFileChooser = FileDialogHelpers.getDataFileChooser();
      File pSelectedFile = 
      FileDialogHelpers.showSaveFileDialog(pFileChooser, this);


      //action after choosing a file
      if (pSelectedFile != null) {

        //save the csv file
        CSVIO.writeCSVFileByDOI(pSelectedFile, m_pDataset, m_pSelectionList.getLocalDOI(0));
      }

      return;

    }

    //10. file menu items - export image
    if (e.getActionCommand().equals("Export image")) {


      //create and modify FileChooser
      JFileChooser pFileChooser = FileDialogHelpers.getImageFileChooser();
      File pSelectedFile = 
      FileDialogHelpers.showSaveFileDialog(pFileChooser, this);
      

      if (pSelectedFile != null) {

        //open export frame
        m_pImageExportFrame.setVisible(true);
        m_pImageExportFrame.setOutputFile(pSelectedFile);
        m_pImageExportFrame.setExportable(m_pParCoordView);
        setEnabled(false);

      }


      return;
    }


  }



  //sets new data and initializes everything new
  private void setNewData(Data pNewDataSet, SelectionList pSelectionList) {
    //1. make a simple check
    if (pNewDataSet == null) return;
    
    //2. set the new data and remove the parcoord view
    m_pDataset = pNewDataSet;

    if (m_pParCoordView != null) {
      remove(m_pParCoordView);
    }

    //3. create SelectionList
    if (pSelectionList == null) {
      m_pSelectionList = new SelectionList(m_pDataset);
    }
    else {
      pSelectionList.setData(m_pDataset);
      m_pSelectionList = pSelectionList;
    }


    //4. create parcoordview
    m_pParCoordView = new ParCoordView(600, 300, m_pSelectionList);
    m_pParCoordView.setLocation(20, 20);
    m_pParCoordView.setData(m_pDataset);

    Vector<IDataSubsetListener> pListeners = new Vector<IDataSubsetListener>();
    pListeners.add(this);
    m_pParCoordView.setInteraction(new ParCoordSelectionInteraction(pListeners, m_pParCoordView, m_pSelectionList));

    //Add Elements to Container
    add(m_pParCoordView);




    //5. init additional member variables

    //5.1. COLORING FRAME: register DataSetListener, SelectionListListener
    m_pColoringFrame = new ColoringFrame(m_pDataset.getDimensionNames());
    m_pDataset.addDataSetListener(m_pColoringFrame);
    m_pSelectionList.addSelectionListListener(m_pColoringFrame);

    //register parallel coordinates as listener of coloringframe
    m_pColoringFrame.addColoringListener(m_pParCoordView);


    //5.2. clustering frame
    m_pClusteringFrame = new ClusteringFrame(m_pDataset);

    //5.3. axis management frame
    m_pAxisManagementFrame = new AxisManagementFrame(m_pDataset);
    m_pDataset.addDataSetListener(m_pAxisManagementFrame);
    m_pAxisManagementFrame.addAxisManagementListener(m_pParCoordView);

    //5.4. image export frame, pca frame, distance frame
    m_pImageExportFrame = new ImageExportFrame(m_pParCoordView, this);
    m_pPCAFrame = new PCAFrame(m_pDataset);
    m_pDistanceFrame = new DistanceFrame(m_pDataset);


    //5.5. create the scatterplot
    ScatterplotView pScatterplotView = new ScatterplotView(400, 400, m_pSelectionList);
    pScatterplotView.setData(m_pDataset);
    pScatterplotView.setInteraction(new ScatterplotSelectionInteraction(pListeners, pScatterplotView, m_pSelectionList));
    m_pScatterplotFrame = new ScatterplotFrame(pScatterplotView, m_pDataset.getDimensionNames());
    m_pColoringFrame.addColoringListener(pScatterplotView);

    //5.6. create the outlier detection
    OutlierDetectionView pOutlierDetectionView = new OutlierDetectionView(400, 400);
    pOutlierDetectionView.setData(m_pDataset);
    pOutlierDetectionView.setInteraction(new OutlierDetectionInteraction(pListeners, pOutlierDetectionView));
    m_pOutlierDetectionFrame = new OutlierDetectionFrame(pOutlierDetectionView);
    m_pColoringFrame.addColoringListener(pOutlierDetectionView);


    //6. repaint all frames
    m_pParCoordView.revalidate();
    m_pParCoordView.repaint();


    m_pSelectionList.fireSelectionListChanged();

    //System.out.println("hallo1");


  }
  
  private void showNotAvailableMessage() {
    String pMessage = "This functionality is currently not available. ";
    pMessage += "In the next version a Math library will be included and this functionality can be used again.";
    String pTitle = "Functionality in this version not available.";
    JOptionPane.showMessageDialog(this, pMessage, pTitle,  JOptionPane.INFORMATION_MESSAGE);
  }


  ////////////////////////////////////////
  //IDATASUBSET LISTENER                //
  ////////////////////////////////////////

  public void setSelectedDataAsData() {
    //1. create backup if necessary
    if (m_pWholeDataset == null) m_pWholeDataset = m_pDataset;

    //2. create a subset of the dataset
    float[] pDOI = m_pSelectionList.getGlobalDOI();
    int iEntryNum = pDOI.length;
    
    //2.1. count the data with doi > 0
    int iCounter = 0;
    for (int i = 0; i < iEntryNum; i++) {
      if (pDOI[i] > 0) iCounter++;
    }
    int iNewEntryNum = iCounter;

    
    //2.2. create the new data and copy dimension by dimension
    Data pNewData = new Data();
    int iDimNum = m_pDataset.getDimNumber();

    for (int iDim = 0; iDim < iDimNum; iDim++) {
      float[] pValues = m_pDataset.getDimensionValues(iDim);

      float[] pNewValues = new float[iNewEntryNum];

      iCounter = 0;
      for (int i = 0; i < iEntryNum; i++) {
        if (pDOI[i] > 0) {
          pNewValues[iCounter] = pValues[i];
          iCounter++;
        }
      }

      pNewData.addDimension(new FloatDataDimension(pNewValues, m_pDataset.getDimensionName(iDim)));

    }


    //3. set the new data
    setNewData(pNewData, null);


  }


  public void setSelectedDataAsData(float[] pDOI) {
    //1. make a simple check
    if (pDOI == null) {
      setSelectedDataAsData();
      return;
    }
  
    //2. create backup if necessary
    if (m_pWholeDataset == null) m_pWholeDataset = m_pDataset;

    //3. create a subset of the dataset
    //3.1. count the number of the data with doi > 0
    int iEntryNum = pDOI.length;
    int iCounter = 0;
    for (int i = 0; i < iEntryNum; i++) {
      if (pDOI[i] > 0) iCounter++;
    }
    int iNewEntryNum = iCounter;

    //3.2. copy the data of interest into a new dataset
    //TO DO: the copy operation should be made in the data object
    Data pNewData = new Data();
    int iDimNum = m_pDataset.getDimNumber();

    for (int iDim = 0; iDim < iDimNum; iDim++) {
      float[] pValues = m_pDataset.getDimensionValues(iDim);
      float[] pNewValues = new float[iNewEntryNum];

      iCounter = 0;
      for (int i = 0; i < iEntryNum; i++) {
        if (pDOI[i] > 0) {
          pNewValues[iCounter] = pValues[i];
          iCounter++;
        }
      }

      pNewData.addDimension(new FloatDataDimension(pNewValues, m_pDataset.getDimensionName(iDim)));

    }


    //4. set the new data
    setNewData(pNewData, null);


    m_pOutlierDetectionFrame.setVisible(false);

  }


  public void resetWholeDatasetAsData() {
    if (m_pWholeDataset != null) setNewData(m_pWholeDataset, m_pSelectionList);
    m_pWholeDataset = null;
  }




  ////////////////////////////////////////
  //KeyListener methods                 //
  ////////////////////////////////////////

  /**
   * Sets a flag in the views to true if a certain key is pressed.
   */
   public void keyPressed(KeyEvent e) {
     if (e.getKeyCode() == KeyEvent.VK_CONTROL) {
       m_pParCoordView.setControlPressed(true);
     }
   }

  /**
   * Sets all flags indicating a pressed key in the views to false.
   */
   public void keyReleased(KeyEvent e) {
     m_pParCoordView.setControlPressed(false);
   }

  /**
   * Unused KeyListener method.
   */
   public void keyTyped(KeyEvent e) {
   }





  ////////////////////////////////////////
  //WindowListener methods              //
  ////////////////////////////////////////
  /**
   * Closing the window terminates the program.
   */
  public void windowClosing(WindowEvent event1) {
    System.exit(0);
  }

  /**
   * Unused WindowListener method.
   */
  public void windowGainedFocus(WindowEvent event2) {}

  /**
   * Unused WindowListener method.
   */
  public void windowClosed(WindowEvent e) {}
  /**
   * Unused WindowListener method.
   */
  public void windowDeactivated(WindowEvent e) {}
  /**
   * Unused WindowListener method.
   */
  public void windowActivated(WindowEvent e) {}
  /**
   * Unused WindowListener method.
   */
  public void windowDeiconified(WindowEvent e) {}
  /**
   * Unused WindowListener method.
   */
  public void windowIconified(WindowEvent e) {}
  /**
   * Unused WindowListener method.
   */
  public void windowOpened(WindowEvent e) {}






}