package data;

/*
 *  Copyright (C) 2018 Juergen Platzer 
 * 
 *  This file is part of JInfoViz.
 *  
 *  JInfoViz is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  JInfoViz is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *  
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with JInfoViz.  If not, see <http://www.gnu.org/licenses/>.
 */
 

/** Class implementing a Selection on one dimension (= axis). An AxisSelection represents
 *  an interval on a dimension. When a data item has a value within this interval on the given
 *  dimension it is fully selected by this AxisSelection. It can also represent soft borders, ie.
 *  margins at the ends of the interval the represent linear selection transition from 1 to 0.
 *  A data item that has a value within a soft border has a selection status of alpha. This alpha
 *  is near to 1 if the value of the data item is near the interval border and near 0 otherwise.
 */
public class AxisSelection extends Selection {

  
  


  /*** member variables ***/
  
  /** ID (with respect to the dataset) of the dimension on which the AxisSelection is defined. */
  private int  m_iDimId;
  
  // selection borders
  private float m_fLowerBorder, m_fUpperBorder;

  // soft selection borders
  private Float m_pSoftLowerBorder, m_pSoftUpperBorder;

  /** Creates a AxisSelection on the dimension specified by dimID with the interval
   *  defined by border1 and border2 and the ID of the view in which it has been created.
   *
   *  @param iDimID ID (with respect to the dataset) of the dimension on which the AxisSelection is defined.
   *  @param fBorder1 a border of the interval of the selection.
   *  @param fBorder2 a border of the interval of the selection.
   *  @param iViewID ID of the view in which the AxisSelection was created.
   */
  public AxisSelection(int iDimID, float fBorder1, float fBorder2, int iViewID) {
    
    //1. set the member variables
    m_iDimId = iDimID;

    if (fBorder1 < fBorder2) {
      m_fLowerBorder = fBorder1;
      m_fUpperBorder = fBorder2;
    }
    else {
      m_fLowerBorder = fBorder2;
      m_fUpperBorder = fBorder1;
    }

    m_pSoftLowerBorder = null;
    m_pSoftUpperBorder = null;

    setViewId(iViewID);


  }


  //////////////////////////////////////////
  // GET METHODS                          //
  //////////////////////////////////////////
  
  /** Returns the ID (with respect to the dataset) of the dimension on which the AxisSelection is defined.
   *
   *  @return the ID (with respect to the dataset) of the dimension on which the AxisSelection is defined.
   */
  public int getDimId() {
    return m_iDimId;
  }
  
  
  /** Returns the lower border of the interval of this AxisSelection.
   *
   *  @return lower border of the interval of this AxisSelection as float.
   */
  public float getLowerBorder() {
    return m_fLowerBorder;
  }

  /** Returns the upper border of the interval of this AxisSelection.
   *
   *  @return upper border of the interval of this AxisSelection as float.
   */
  public float getUpperBorder() {
    return m_fUpperBorder;
  }

  /** Returns the lower soft border of this AxisSelection. If no lower soft border is defined
   *  the lower border of the interval is returned.
   *
   *  @return lower soft border of the interval of this AxisSelection as float.
   */
  public float getSoftLowerBorder() {
    if (m_pSoftLowerBorder == null) return m_fLowerBorder;

    return m_pSoftLowerBorder.floatValue();
  }


  /** Returns the upper soft border of this AxisSelection. If no upper soft border is defined
   *  the upper border of the interval is returned.
   *
   *  @return upper soft border of the interval of this AxisSelection as float.
   */
  public float getSoftUpperBorder() {
    if (m_pSoftUpperBorder == null) return m_fUpperBorder;

    return m_pSoftUpperBorder.floatValue();
  }



  /** Returns the degree of interest for a given array of values. The degree of interest for a
   *  value is a float between 1 and 0, where 1 means fully selected and 0 means not selected.
   *
   *  @param pValues An array of float values representing the data points.
   *
   *  @return degree of interest of data points as float array.
   */
  public float[] getDOI(float[] pValues) {
    //1. make a simple check
    if (pValues == null) return null;

    //2. create a DOI array and run through all values
    //2.1. create the DOI array
    int iEntryNum = pValues.length;
    float[] pDOI = new float[iEntryNum];
    
    //2.2. make the soft border calculations before the loop
    boolean bHasSoftLowerBorder = hasSoftLowerBorder();
    float fSoftLowerBorder = -1.0f;
    float fSoftLowerBorderInterval = -1.0f;
    if (bHasSoftLowerBorder) {
      fSoftLowerBorder = m_pSoftLowerBorder.floatValue();
      fSoftLowerBorderInterval = (m_fLowerBorder - fSoftLowerBorder);
    }
    
    
    boolean bHasSoftUpperBorder = hasSoftUpperBorder();
    float fSoftUpperBorder = -1.0f;
    float fSoftUpperBorderInterval = -1.0f;
    if (bHasSoftUpperBorder) {
      fSoftUpperBorder = m_pSoftUpperBorder.floatValue();
      fSoftUpperBorderInterval = (fSoftUpperBorder - m_fUpperBorder);
    }
    
    
    

    for (int i = 0; i < iEntryNum; i++) {
      float pCurrVal = pValues[i];

      //within hard borders
      if ((pCurrVal > m_fLowerBorder) && (pCurrVal < m_fUpperBorder)) {
        pDOI[i] = 1.0f;
      }

      //lower than lower hard border
      else if (pCurrVal < m_fLowerBorder) {

        //within lower soft border
        if (bHasSoftLowerBorder && (pCurrVal > fSoftLowerBorder)) {
          pDOI[i] = (pCurrVal - fSoftLowerBorder) / fSoftLowerBorderInterval;
        }
        else {
          pDOI[i] = 0.0f;
        }

      }

      //higher than upper hard border
      else {

        //within upper lower border
        if (bHasSoftUpperBorder && (pCurrVal < fSoftUpperBorder)) {
          pDOI[i] = (fSoftUpperBorder - pCurrVal) / fSoftUpperBorderInterval;
        }
        else {
          pDOI[i] = 0.0f;
        }
      }


    }


    return pDOI;


  }


  //////////////////////////////////////////
  // SET METHODS                          //
  //////////////////////////////////////////
  /** Sets the lower border of the interval of this AxisSelection. The lower border must have a
   *  smaller value than the current upper border of the interval.
   *
   *  @param fValue The value of the lower border.
   *
   *  @return The float value of the lower border.
   */
  public boolean setLowerBorder(float fValue) {
    if (fValue <= m_fUpperBorder) {
      m_fLowerBorder = fValue;
      selectionChanged();
      return true;
    }
    return false;
  }


  /** Sets the upper border of the interval of this AxisSelection. The upper border must have a
   *  higher value than the current lower border of the interval.
   *
   *  @param fValue The value of the upper border.
   *
   *  @return The float value of the upper border.
   */
  public boolean setUpperBorder(float fValue) {
    if (fValue >= m_fLowerBorder) {
      m_fUpperBorder = fValue;
      selectionChanged();
      return true;
    }
    return false;
  }


  /** Sets the soft lower border of this AxisSelection. The soft lower border must have a
   *  smaller value than the current lower border of the interval.
   *
   *  @param fValue The value of the soft lower border.
   *
   */
  public boolean setSoftLowerBorder(float fValue) {
    if (fValue <= m_fLowerBorder) {
      m_pSoftLowerBorder = new Float(fValue);
      selectionChanged();
      return true;
    }
    return false;
  }

  /** Sets the soft upper border of this AxisSelection. The soft upper border must have a
   *  higher value than the current upper border of the interval.
   *
   *  @param fValue The value of the soft upper border.
   */
  public boolean setSoftUpperBorder(float fValue) {
    if (fValue >= m_fUpperBorder) {
      m_pSoftUpperBorder = new Float(fValue);
      selectionChanged();
      return true;
    }
    return false;
  }


  //////////////////////////////////////////
  // CHECK STATE METHODS                  //
  //////////////////////////////////////////

  /** Returns a boolean indicating if this AxisSelection has a soft lower border.
   *
   *  @return <code>true</code> if there is a soft lower border.
   */
  public boolean hasSoftLowerBorder() {
    return (m_pSoftLowerBorder != null);
  }

  /** Returns a boolean indicating if this AxisSelection has a soft upper border.
   *
   *  @return <code>true</code> if there is a soft upper border.
   */
  public boolean hasSoftUpperBorder() {
    return (m_pSoftUpperBorder != null);
  }




}