package data;

/*
 *  Copyright (C) 2018 Juergen Platzer 
 * 
 *  This file is part of JInfoViz.
 *  
 *  JInfoViz is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  JInfoViz is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *  
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with JInfoViz.  If not, see <http://www.gnu.org/licenses/>.
 */

import java.io.*;
import java.util.*;


/** 
 *   Reading data from a csv format and writing data to the csv format.
 */
public class CSVIO {

  public static final String SEPARATOR = ";";


  /** Reads a file in csv format and creates a data object. The csv format has to contain
   *  the dimension names in the first line. The values should be separated by a <code>;</code>.
   *  Missing values are not handled.
   *
   *  @param pFile The file object from which the data should be read.
   *
   *  @return The data object that is created from the data in the given csv file.
   */
  public static Data readCSVFile(File pFile) {
    //1. make a simple check
    if (pFile == null) return null;

    
    try {

      //2. create the filereader
      BufferedReader pBufferedReader = new BufferedReader(new FileReader(pFile));


      //3. read first line containing the column names
      String pLine = pBufferedReader.readLine();
      String[] pColNames = pLine.split(SEPARATOR);
      int iColNum = pColNames.length;


      //4. create an Array of Vectors collecting the data values
      Vector<Vector<String>> pColValues = new Vector<Vector<String>>();
      for (int i = 0; i < iColNum; i++) {
        pColValues.add(new Vector<String>());
      }


      //5. fill up the Vectors with the values in String format
      pLine = pBufferedReader.readLine();
      int iRowNum = 0;
      int iCounter = 0;
      String[] pEntryValues = null;
      
      while (pLine != null) {

        pEntryValues = pLine.split(SEPARATOR);
      
        for (int i = 0; i < iColNum; i++) {
          pColValues.get(i).add(pEntryValues[i]);
        }

        iRowNum++;
        pLine = pBufferedReader.readLine();


        if(iCounter%100 == 0) System.out.println("Line " + iCounter + " read!");
        iCounter++;


      }


      //6. create the data and the datadimensions
      Data pData = new Data();
      for (int i = 0; i < iColNum; i++) {
        Vector<String> pCurrColValues = pColValues.get(i);

        //6.1. decide if float or stringformat of the datavalues
        boolean bIsString = false;
        try {
          float fTest = Float.parseFloat(pCurrColValues.get(0));
        }
        catch(Exception e) {
          bIsString = true;
        }


        //6.2. create the string format column
        if (bIsString) {
          DataDimension pDimension = new StringDataDimension(pCurrColValues, pColNames[i]);
          pData.addDimension(pDimension);

        }
        else {
          float[] pValues = new float[iRowNum];

          for (int j = 0; j < iRowNum; j++) {
            try {
              pValues[j] = Float.parseFloat(pCurrColValues.get(j));
            }
            catch(Exception e) {
              pValues[j] = Float.NaN;
            }
          }

          DataDimension pDimension = new FloatDataDimension(pValues, pColNames[i]);
          pData.addDimension(pDimension);
        }



      }

      return pData;
    }
    catch (Exception ex) {
      ex.printStackTrace();
      return null;
    }


  }


  /** Writes data items of the given data object with a degree of interest greater than 0 into the
   *  given file in csv format. The csv format has to contains
   *  the dimension names in the first line. The values are separated by a <code>;</code>.
   *  Missing values are not handled.
   *
   *  @param pFile The file object into which the data should be written.
   *  @param pData Data object that contains the data items.
   *  @param pDOI float array holding a degree of interest value for each data item.
   *
   */
  public static void writeCSVFileByDOI(File pFile, Data pData, float[] pDOI) {
    
    //1. make some checks
    if (pFile == null) return;
    if (pData == null) return;
    if (pDOI == null) return;

    
    try {
      //2. create the filereader
      BufferedWriter pWriter = new BufferedWriter(new FileWriter(pFile));

      //3. write the header
      String pFirstLine = "";
      int iDimNum = pData.getDimNumber();
      
      for (int i = 0; i < pData.getDimNumber(); i++) {
        pFirstLine += pData.getDimensionName(i);

        if (i < iDimNum - 1) {
          pFirstLine += SEPARATOR;
        }
      }

      pWriter.write(pFirstLine);
      pWriter.newLine();

      
      //4. write the data
      int iEntryNum = pDOI.length;
      for (int i = 0; i < iEntryNum; i++) {
        if (pDOI[i] == 0) continue;

        String pLine = pData.itemToCSVString(i, SEPARATOR);
        pWriter.write(pLine);
        pWriter.newLine();
      }

      pWriter.close();
    }
    catch (Exception e) {
      e.printStackTrace();
    }

  }

}