package data;

/*
 *  Copyright (C) 2018 Juergen Platzer 
 * 
 *  This file is part of JInfoViz.
 *  
 *  JInfoViz is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  JInfoViz is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *  
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with JInfoViz.  If not, see <http://www.gnu.org/licenses/>.
 */


import java.util.Vector;


/** Class implementing a dimension of the data that is holding float data.
 */
public class FloatDataDimension extends DataDimension {

  private float[]   m_pValues;              //data values
  private String    m_pName;


  //statistics
  private float     m_fDimMin, m_fDimMax;         //minimum and maximum values
  private int       m_iDimMinPos, m_iDimMaxPos;   //minimum and maximum positions
  private float     m_fDimMean;                   //mean value of the dimension


  //////////////////////////////////////////
  // CONSTRUCTORS                         //
  //////////////////////////////////////////

  /** Creates a data dimension with numerical values.
   *
   *  @param values float array holding the values of the dimension.
   *  @param name String representing the name of the dimension.
   */
  public FloatDataDimension(float[] pValues, String pName) {

    //1. make a simple check
    if (pValues == null) {
      pValues = new float[1];
      pValues[0] = 0.0f;
    }
    
    
    //2. set the members
    m_pName = pName; //needs to be before the set values, because the name is communicated to the father class
    setValues(pValues);
    


  }

  /////////////////////////////////////
  // IMPLEMENTATION OF THE INTERFACE //
  /////////////////////////////////////

  
  
  /** Sets new values to the dimension.
   *
   * @param pValues float array holding a value for each data item.
   *
   */
  public void setValues(float[] pValues) {
  
    //1. make a simple check
    if (pValues == null) return;

    //2. copy the data and calculate the statistics
    //2.1. init the variables
    int iEntryNum = pValues.length;
    m_pValues = new float[iEntryNum];
    
    m_fDimMin = Float.POSITIVE_INFINITY;
    m_fDimMax = Float.NEGATIVE_INFINITY;
    m_iDimMinPos = 0;
    m_iDimMaxPos = 0;


    float fSum = 0.0f;
    float fCurrValue = 0.0f;
    
    //2.2. do the copy and the statistics calculation
    for (int i = 0; i < iEntryNum; i++) {
      //copy
      fCurrValue = pValues[i];
      m_pValues[i] = fCurrValue;
      
      //mean
      fSum += fCurrValue;

      //minimum
      if (fCurrValue < m_fDimMin) {
        m_fDimMin = fCurrValue;
        m_iDimMinPos = i;
      }

      //maximum
      if (fCurrValue > m_fDimMax) {
        m_fDimMax = fCurrValue;
        m_iDimMaxPos = i;
      }
    }

    m_fDimMean = fSum/(float)iEntryNum;
  
  
    //3. set the statistics data in the abstract class
    setDimensionInfo(m_pName, iEntryNum, m_fDimMin, m_fDimMax, m_iDimMinPos, m_iDimMaxPos, m_fDimMean);
    
  }

  
  

  /** Returns the values of the dimension as float arry.
   *
   * @return float array holding a value for each data item.
   */
  public float[] getValues() {
    return m_pValues;
  }

  


  /** Returns the String representation of the value of the specified data item in this dimension.
   *
   * @return String representation of the value of the specified data item in this dimension.
   */
  public String getStringValue(int iEntryId) {
    if (iEntryId < 0) return null;
    if (iEntryId >= m_pValues.length) return null;

    return Float.toString(m_pValues[iEntryId]);
  }



}