package data;

/*
 *  Copyright (C) 2018 Juergen Platzer 
 * 
 *  This file is part of JInfoViz.
 *  
 *  JInfoViz is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  JInfoViz is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *  
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with JInfoViz.  If not, see <http://www.gnu.org/licenses/>.
 */


/** Class implementing a Selection on one two dimensions (= axes). An ScatterplotSelection represents
 *  an intervall on each of the two dimensions. When a data item has a value within both of these
 *  intervalls on the given dimensions it is fully selected by this ScatterplotSelection.
 */
public class ScatterplotSelection extends Selection {

  /** IDs (with respect to the dataset) of the dimensions on which the ScatterplotSelection is defined. */
  public int  m_iXdimID, m_iYdimID;


  /*** member variables ***/
  private float m_fXMin, m_fYMin, m_fXMax, m_fYMax;


  /** Creates a ScatterplotSelection on the dimensions specified by iXdimID and iYdimID with the intervall
   *  defined by fX1 and fX2 on the iXdimID dimension and defined by fY1 and fY2 on the iYdimID dimension.
   *  viewID indicates the ID of the view in which the selection has been created.
   *
   *  @param iXdimID ID (with respect to the dataset) of the x dimension on which the ScatterplotSelection is defined.
   *  @param yDimID ID (with respect to the dataset) of the y dimension on which the ScatterplotSelection is defined.
   *  @param fX1 a border of the intervall of the selection on the x dimension.
   *  @param fX2 a border of the intervall of the selection on the x dimension.
   *  @param fY1 a border of the intervall of the selection on the y dimension.
   *  @param fY2 a border of the intervall of the selection on the y dimension.
   *  @param iViewId ID of the view in which the AxisSelection was created.
   */
  public ScatterplotSelection(
    int iXdimID, 
    int iYdimID, 
    float fX1, 
    float fX2, 
    float fY1, 
    float fY2, 
    int iViewId
  ) {

    m_iXdimID = iXdimID;
    m_iYdimID = iYdimID;


    //minPoint
    if (fX1 < fX2) {
      m_fXMin = fX1;
      m_fXMax = fX2;
    }
    else {
      m_fXMin = fX2;
      m_fXMax = fX1;
    }


    if (fY1 < fY2) {
      m_fYMin = fY1;
      m_fYMax = fY2;
    }
    else {
      m_fYMin = fY2;
      m_fYMax = fY1;
    }


    setViewId(iViewId);


  }


  //////////////////////////////////////////
  // GET METHODS                          //
  //////////////////////////////////////////

  /** Returns the lower border of the intervall on the x dimension.
   *
   *  @return lower border of the intervall on the x dimension as float.
   */
  public float getLowerXBorder() {
    return m_fXMin;
  }

  /** Returns the upper border of the intervall on the x dimension.
   *
   *  @return upper border of the intervall on the x dimension as float.
   */
  public float getUpperXBorder() {
    return m_fXMax;
  }

  /** Returns the lower border of the intervall on the y dimension.
   *
   *  @return lower border of the intervall on the y dimension as float.
   */
  public float getLowerYBorder() {
    return m_fYMin;
  }

  /** Returns the upper border of the intervall on the y dimension.
   *
   *  @return upper border of the intervall on the y dimension as float.
   */
  public float getUpperYBorder() {
    return m_fYMax;
  }




  /** Returns the degree of interest for a given array of values for the x dimension and for the y dimension.
   *  The degree of interest for a
   *  value is a float between 1 and 0, where 1 means fully selected and 0 means not selected.
   *
   *  @return degree of interest of data points as float array.
   */
  public float[] getDOI(float[] pXValues, float[] pYValues) {
    if (pXValues == null) return null;
    if (pYValues == null) return null;

    if (pXValues.length != pYValues.length) return null;

    int iEntryNum = pXValues.length;
    float[] pDOI = new float[iEntryNum];

    for (int i = 0; i < iEntryNum; i++) {


      //within hard borders
      float fCurrXValue = pXValues[i];
      float fCurrYValue = pYValues[i];
      
      if ((fCurrXValue >= m_fXMin) && (fCurrXValue <= m_fXMax) &&
          (fCurrYValue >= m_fYMin) && (fCurrYValue <= m_fYMax)) {
        pDOI[i] = 1.0f;
      }
      else {
        pDOI[i] = 0.0f;
      }


    }


    return pDOI;


  }


  //////////////////////////////////////////
  // SET METHODS                          //
  //////////////////////////////////////////
  /** Sets the lower border of the intervall on the x dimension. The lower border must have a
   *  smaller value than the current upper border of the intervall on the x dimension.
   *
   *  @param fValue The value of the lower border on the x dimension.
   */
  public boolean setLowerXBorder(float fValue) {
    if (fValue <= m_fXMax) {
      m_fXMin = fValue;
      selectionChanged();
      return true;
    }
    return false;
  }

  /** Sets the upper border of the intervall on the x dimension. The upper border must have a
   *  higher value than the current lower border of the intervall on the x dimension.
   *
   *  @param fValue The value of the upper border on the x dimension.
   */
  public boolean setUpperXBorder(float fValue) {
    if (fValue >= m_fXMin) {
      m_fXMax = fValue;
      selectionChanged();
      return true;
    }
    return false;
  }

  /** Sets the lower border of the intervall on the y dimension. The lower border must have a
   *  smaller value than the current upper border of the intervall on the y dimension.
   *
   *  @param value The value of the lower border on the y dimension.
   */
  public boolean setLowerYBorder(float fValue) {
    if (fValue <= m_fYMax) {
      m_fYMin = fValue;
      selectionChanged();
      return true;
    }
    return false;
  }


  /** Sets the upper border of the intervall on the y dimension. The upper border must have a
   *  higher value than the current lower border of the intervall on the y dimension.
   *
   *  @param value The value of the upper border on the y dimension.
   */
  public boolean setUpperYBorder(float fValue) {
    if (fValue >= m_fYMin) {
      m_fYMax = fValue;
      selectionChanged();
      return true;
    }
    return false;
  }




  //////////////////////////////////////////
  // CHECK STATE METHODS                  //
  //////////////////////////////////////////

  /** Returns a boolean indicating if this AxisSelection has a soft lower border.
   *
   *  @return <code>true</code> if there is a soft lower border.
   */
  public boolean hasSoftLowerBorder() {
    return false;
  }

  /** Returns a boolean indicating if this AxisSelection has a soft upper border.
   *
   *  @return <code>true</code> if there is a soft upper border.
   */
  public boolean hasSoftUpperBorder() {
    return false;
  }




}