package gui;

/*
 *  Copyright (C) 2018 Juergen Platzer 
 * 
 *  This file is part of JInfoViz.
 *  
 *  JInfoViz is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  JInfoViz is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *  
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with JInfoViz.  If not, see <http://www.gnu.org/licenses/>.
 */

import java.awt.Component;
import java.io.File;
import javax.swing.JFileChooser;
import javax.swing.filechooser.FileNameExtensionFilter;
import javax.swing.filechooser.FileFilter;

public class FileDialogHelpers {

  /**** constants ****/
  public static final String FILE_EXTENSION_JPG = "JPG";
  public static final String FILE_EXTENSION_PNG = "PNG";
  public static final String FILE_EXTENSION_BMP = "BMP";
  public static final String FILE_EXTENSION_GIF = "GIF";
  public static final String FILE_EXTENSION_CSV = "CSV";

  
  /////////////////////////////
  // FILE FILTER METHODS     //
  /////////////////////////////
  public static FileFilter getFileExtensionFilter(String pFileExtension) {
    //1. make a simple check
    if (pFileExtension == null) return null;
    
    //2. create the file extension filter
    if (pFileExtension.equals(FILE_EXTENSION_JPG)) {
      return new FileNameExtensionFilter("JPEG file", "jpg", "jpeg");
    }
    if (pFileExtension.equals(FILE_EXTENSION_PNG)) {
      return new FileNameExtensionFilter("Portable Network Graphic file", "png");
    }
    if (pFileExtension.equals(FILE_EXTENSION_BMP)) {
      return new FileNameExtensionFilter("Bitmap file", "bmp");
    }
    if (pFileExtension.equals(FILE_EXTENSION_GIF)) {
      return new FileNameExtensionFilter("GIF file", "gif");
    }
    if (pFileExtension.equals(FILE_EXTENSION_CSV)) {
      return new FileNameExtensionFilter("Comma separated values file", "csv");
    }
    
    return null;
    
  }
  
  
  ////////////////////////////
  // FILE CHOOSER METHODS   //
  ////////////////////////////
  public static JFileChooser getDataFileChooser() {
    JFileChooser pFileChooser = new JFileChooser();
    pFileChooser.addChoosableFileFilter(getFileExtensionFilter(FILE_EXTENSION_CSV));
    pFileChooser.setAcceptAllFileFilterUsed(false);
    
    return pFileChooser;
  }
  
  public static JFileChooser getImageFileChooser() {
    JFileChooser pFileChooser = new JFileChooser();
    
    FileFilter pDefaultFileFilter = getFileExtensionFilter(FILE_EXTENSION_PNG);
    pFileChooser.addChoosableFileFilter(pDefaultFileFilter);
    pFileChooser.addChoosableFileFilter(getFileExtensionFilter(FILE_EXTENSION_JPG));
    pFileChooser.addChoosableFileFilter(getFileExtensionFilter(FILE_EXTENSION_GIF));
    pFileChooser.addChoosableFileFilter(getFileExtensionFilter(FILE_EXTENSION_BMP));
    pFileChooser.setFileFilter(pDefaultFileFilter);
    pFileChooser.setAcceptAllFileFilterUsed(false);
    
    return pFileChooser;
  }
  
  
  //////////////////////////////////
  // FILE DIALOG METHODS          //
  //////////////////////////////////
  public static File showOpenFileDialog(JFileChooser pFileChooser, Component pParent) {
    //1. make a simple check
    if (pFileChooser == null) return null;
    
    //2. open the dialog and return the file afterwards
    int iReturnVal = pFileChooser.showOpenDialog(pParent);
    if(iReturnVal == JFileChooser.APPROVE_OPTION) {
      return pFileChooser.getSelectedFile();
    }
    return null;
  }
  
  public static File showSaveFileDialog(JFileChooser pFileChooser, Component pParent) {
    //1. make a simple check
    if (pFileChooser == null) return null;
    
    //2. open the dialog and return the file afterwards
    int iReturnVal = pFileChooser.showSaveDialog(pParent);
    if(iReturnVal == JFileChooser.APPROVE_OPTION) {
      File pFile = pFileChooser.getSelectedFile();
      
      //check if there is a valid file extension
      boolean bValidFileExtension = false;
      String pFileName = pFile.getName();
      int iFileNameLength = pFileName.length();
      if (iFileNameLength > 4) {
        String pFileExtension = pFileName.substring(pFileName.length()-4);
        if (pFileExtension.equalsIgnoreCase(".jpg")) bValidFileExtension = true;
        if (pFileExtension.equalsIgnoreCase(".gif")) bValidFileExtension = true;
        if (pFileExtension.equalsIgnoreCase(".bmp")) bValidFileExtension = true;
        if (pFileExtension.equalsIgnoreCase(".png")) bValidFileExtension = true;
        if (pFileExtension.equalsIgnoreCase(".csv")) bValidFileExtension = true;
      }
      
      //if there is no valid file extension -> try to set one
      if (!bValidFileExtension) {
        FileFilter pFilter = pFileChooser.getFileFilter();
        if (pFilter instanceof FileNameExtensionFilter) {
          String[] pExtensions = ((FileNameExtensionFilter)pFilter).getExtensions();
          if ((pExtensions != null) && (pExtensions.length > 0)) {
            String pAbsoluteFileName = pFile.getAbsolutePath();
            pFile = new File(pAbsoluteFileName + "." + pExtensions[0]);
          }
        }
      }
      
      return pFile;
      
      
    }
    return null;
  }
  


}