package tools;

/*
 *  Copyright (C) 2018 Juergen Platzer 
 * 
 *  This file is part of JInfoViz.
 *  
 *  JInfoViz is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  JInfoViz is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *  
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with JInfoViz.  If not, see <http://www.gnu.org/licenses/>.
 */


/** Class implementing cluster centers. For a k means clustering this corresponds
 *  to a <code>k</code> x <code>dimNumber</code> array of float values.
 */
public class ClusterCenters {

  /** member variables **/
  private int m_iClusterNum;     //number of clusters
  private int m_iDimensionNum;   //number of dimensions

  private float[][] m_pCenters; //2 dim float array holding the centers (m_iClusterNum x m_iDimensionNum)

  //////////////////////////////////////////
  // CONSTRUCTOR                          //
  //////////////////////////////////////////
  /** Creates <code>clusternumber</code> cluster centers that are represented by float
   *  arrays of length <code>dimnumber</code>. The cluster centers initially contain only zeros.
   *
   *  @param clusternumber number of cluster centers.
   *  @param dimnumber number of dimensions.
   *
   */
  public ClusterCenters(int iClusterNumber, int iDimensionNum) {

    //1. check parameters
    if (iClusterNumber < 1) return;
    if (iDimensionNum < 1) return;

    //2. set member variables
    m_iClusterNum = iClusterNumber;
    m_iDimensionNum = iDimensionNum;

    m_pCenters = new float[m_iClusterNum][m_iDimensionNum];
    for (int iCluster = 0; iCluster < m_iClusterNum; iCluster++) {
      for (int iDim = 0; iDim < m_iDimensionNum; iDim++) {
        m_pCenters[iCluster][iDim] = 0.0f;
      }
    }

  }


  //////////////////////////////////////////
  // GET METHODS                          //
  //////////////////////////////////////////
  /** Returns the value of the specified cluster center in the specified dimension.
   *
   *  @param clustID ID of the cluster.
   *  @param dimID ID of the dimension.
   *
   *  @return Value of the specified cluster center in the specified dimension.
   */
  public float getCenterValue(int iClusterId, int iDimId) {
    if (iClusterId < 0) return 0.0f;
    if (iClusterId >= m_iClusterNum) return 0.0f;

    if (iDimId < 0) return 0.0f;
    if (iDimId >= m_iDimensionNum) return 0.0f;

    return m_pCenters[iClusterId][iDimId];

  }

  //////////////////////////////////////////
  // SET METHODS                          //
  //////////////////////////////////////////
  /** Sets the value of the specified cluster center in the specified dimension.
   *
   *  @param iClusterId ID of the cluster.
   *  @param iDimId ID of the dimension.
   *  @param value new value of the specified cluster center in the specified dimension.
   *
   */
  public void setCenterValue(int iClusterId, int iDimId, float fValue) {
    if (iClusterId < 0) return;
    if (iClusterId >= m_iClusterNum) return;

    if (iDimId < 0) return;
    if (iDimId >= m_iDimensionNum) return;

    m_pCenters[iClusterId][iDimId] = fValue;

  }



  /** Returns a String representation of all cluster centers.
   *
   *  @return String representation of all cluster centers.
   */
  public String toString() {
    String pText = m_iClusterNum + " clusters, " + m_iDimensionNum + " dimensions\n";
    pText += "_____________________\n";

    for (int iCluster = 0; iCluster < m_iClusterNum; iCluster++) {
      pText += "Cluster " + iCluster + "\n";

      for (int iDim = 0; iDim < m_iDimensionNum; iDim++) {
        pText += "Dimension " + iDim + ": " + m_pCenters[iCluster][iDim] + "\n";
      }
      pText += "_____________________\n";
    }

    return pText;


  }


}