package tools;

/*
 *  Copyright (C) 2018 Juergen Platzer 
 * 
 *  This file is part of JInfoViz.
 *  
 *  JInfoViz is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  JInfoViz is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *  
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with JInfoViz.  If not, see <http://www.gnu.org/licenses/>.
 */



import data.Data;
import data.DataDimension;
import data.FloatDataDimension;
import listeners.GlobalConstants;



import java.util.Random;


/** class implementing the K means clustering **/
public class KMeans implements GlobalConstants {



  /*** member variables ***/
  private Data    m_pData;       //holds the data that should be clustered
  private int     m_iClusterNum;   //how many clusters should be created
  private int     m_iMaxIterations;    //how many iteration should be made when the algorithm does not converge
  private float   m_fMinimumUpdate;  //if no updates of the centers higher than m_fMinimumUpdate are made, then the algorithm stops
  private int   m_iCurrentIteration;  //holds the current iteration number

  private float[] m_pDimensionWeights; //weights for the dimensions

  private ClusterCenters m_pClusterCenters; //cluster centers

  //TO DO: make the cluster label an integer array
  private float[] m_pClusterLabels;      //array holding the clusterlabel for each entry
  private float[] m_pClusterDistances;   //array holding the distances for each entry to its cluster center
  private int[]   m_pClusterPopulations; //array holding the number of items of each cluster


  ///////////////////////////////
  // CONSTRUCTOR               //
  ///////////////////////////////
  public KMeans (Data pData, int iClusterNum, int iMaxIterations, int iMinimumUpdate, float[] pDimensionWeights) {

    //1. set member variables
    m_pData = pData;
    m_iClusterNum = iClusterNum;
    m_iMaxIterations = iMaxIterations;
    m_fMinimumUpdate = (float)iMinimumUpdate/100;

    m_iCurrentIteration = 0;

    m_pDimensionWeights = pDimensionWeights;





    //2. do the clustering
    m_pClusterCenters = defineNewClusterCenters();

    //System.out.println(m_pClusterCenters.toString());

    cluster();


  }


  ///////////////////////////////////
  // PRIVATE CALCULATION METHODS   //
  ///////////////////////////////////
  //defines cluster centers
  private ClusterCenters defineNewClusterCenters() {

    //1. create the centers
    int iDimNum = m_pData.getDimNumber();
    ClusterCenters pCenters = new ClusterCenters(m_iClusterNum, iDimNum);



    //2. select randomly k samples indices. every sample is used once.
    int[] pRandomIndices = new int[m_iClusterNum];
    Random pRandom = new Random();
    int iEntryNum = m_pData.getEntryNum();

    for (int iCluster = 0; iCluster < m_iClusterNum; iCluster++) {

      boolean bNotNew = true;  //indicates if index was already used
      int iRandomNumber = 0;

      //fetch a random index until an index is found that is not used yet.
      while (bNotNew) {

        iRandomNumber = pRandom.nextInt(iEntryNum);
        bNotNew = false;

        //check if index already used
        for (int iUsed = 0; iUsed < iCluster; iUsed++) {
          if (pRandomIndices[iUsed] == iRandomNumber) {
            bNotNew = true;
            break;
          }
        }
      }



      pRandomIndices[iCluster] = iRandomNumber;
    }


    //3. set cluster centers
    for (int iDim = 0; iDim < iDimNum; iDim++) {

      //3.1. load the values of the current dimension
      float[] pValues = m_pData.getDimensionValues(iDim);
      float   fMin = m_pData.getDimensionMinimum(iDim);
      float   fMax = m_pData.getDimensionMaximum(iDim);
      float   fRange = fMax - fMin;

      //to avoid NaNs: the cluster centers keep their zero - entries created by the constructor
      if (fRange == 0) continue;

      //3.2. set for all clusters the values of this dimension
      for (int iCluster = 0; iCluster < m_iClusterNum; iCluster++) {

        //get the dimension value and normalize it
        float fCurrValue = pValues[pRandomIndices[iCluster]];
        fCurrValue = (fCurrValue - fMin)/fRange;


        pCenters.setCenterValue(iCluster, iDim, fCurrValue);

      }


    }


    //return the centers
    return pCenters;



  }


  //makes the k means clustering
  private void cluster() {

    //1. create an array of counting the clusterpopulation
    m_pClusterPopulations = new int[m_iClusterNum];
    for (int iCluster = 0; iCluster < m_iClusterNum; iCluster++) {
      m_pClusterPopulations[iCluster] = 0;
    }



    //2. calculate an array holding the distance of each data point to each cluster
    float[][] pDistanceTable = calculateDistanceTable();




    //3. calculate the label for each entry and its distance to its cluster center
    int iEntryNum = m_pData.getEntryNum();
    m_pClusterLabels = new float[iEntryNum];
    m_pClusterDistances = new float[iEntryNum];

    //run through all entries
    for (int i = 0; i < iEntryNum; i++) {

      //check which cluster center has the minimum distance
      float fMinDist = pDistanceTable[0][i];
      int iBestClusterId = 0;

      for (int iCluster = 1; iCluster < m_iClusterNum; iCluster++) {
        float fCurrDistance = pDistanceTable[iCluster][i];
        if (fCurrDistance < fMinDist) {
          fMinDist = fCurrDistance;
          iBestClusterId = iCluster;
        }
      }

      m_pClusterLabels[i] = iBestClusterId;
      m_pClusterDistances[i] = fMinDist;

      m_pClusterPopulations[iBestClusterId]++;
    }



    //4. calculate new cluster centers
    //   (but only the dimensions with weight > 0. The others can be calculated after the
    //    clustering is finished)

    ClusterCenters pNewCenters = calculateUsedDimensionsOfClusterCenters();



    //5. calculate the distances between the old and new cluster centers
    //5.1. init the distance array with zeros
    float[] pCenterDistances = new float[m_iClusterNum];
    for (int iCluster = 0; iCluster < m_iClusterNum; iCluster++) {
      pCenterDistances[iCluster] = 0.0f;
    }

    //5.2. calculate the update distances (add for each relevant dimension the squared difference)
    int iDimNum = m_pData.getDimNumber();
    for (int iDim = 0; iDim < iDimNum; iDim++) {

      //consider only dimensions with a weight > 0
      if (m_pDimensionWeights[iDim] <= 0) continue;

      //load the values of the current dimension
      float   fMin = m_pData.getDimensionMinimum(iDim);
      float   fMax = m_pData.getDimensionMaximum(iDim);
      float   fRange = fMax - fMin;
      if (fRange == 0) continue;

      for (int iCluster = 0; iCluster < m_iClusterNum; iCluster++) {
        float fCurrDistance = pNewCenters.getCenterValue(iCluster, iDim) - 
        m_pClusterCenters.getCenterValue(iCluster, iDim);

        pCenterDistances[iCluster] += fCurrDistance*fCurrDistance;
      }
    }

    //5.3. check if an update happened that was bigger than the minimum update limit
    boolean bBigUpdateHappened = false;
    for (int iCluster = 0; iCluster < m_iClusterNum; iCluster++) {
      if (pCenterDistances[iCluster] > m_fMinimumUpdate) {
        //System.out.println("cluster: " + clust + "update distance " + pCenterDistances[clust]);
        bBigUpdateHappened = true;
        break;
      }
    }



    //6. update the cluster centers
    m_pClusterCenters = pNewCenters;

    System.out.println("============================================");
    System.out.println("Iteration: " + m_iCurrentIteration);
    System.out.println("============================================");
    System.out.println(m_pClusterCenters.toString());




    //7. decide if we should run another iteration
    if (!bBigUpdateHappened) {

      finishClustering();

      //check the coloring panel, why are the clustering distances not there
      return;
    }

    m_iCurrentIteration++;
    if (m_iCurrentIteration >= m_iMaxIterations) {
      finishClustering();

      //check the coloring panel, why are the clustering distances not there
      return;
    }

    //8. if no criteria stopped the clustering -> go into the next iteration
    cluster();

  }


  //calculates an array (clusterNumber x entryNumber)
  //in cell (i,j) the distance of item j to cluster i is saved
  //the dimension weights are used to weigh down dimensions with low weights
  private float[][] calculateDistanceTable() {


    //1. create an array holding the distance of each data point to each cluster
    int iEntryNum = m_pData.getEntryNum();
    float[][] pDistanceTable = new float[m_iClusterNum][iEntryNum];
    for (int iCluster = 0; iCluster < m_iClusterNum; iCluster++) {
      for (int i = 0; i < iEntryNum; i++) {
        pDistanceTable[iCluster][i] = 0.0f;
      }
    }
    
    //System.out.println("1");


    //2. calculate the whole distance table
    int iDimNum = m_pData.getDimNumber();
    for (int iDim = 0; iDim < iDimNum; iDim++) {

      //2.1. consider only dimensions with a weight > 0
      float fDimensionWeight = m_pDimensionWeights[iDim];
      if (fDimensionWeight <= 0) continue;
      //System.out.println("2.1");

      //2.2. load the values of the current dimension
      float[] pValues = m_pData.getDimensionValues(iDim);
      float   fMin = m_pData.getDimensionMinimum(iDim);
      float   fMax = m_pData.getDimensionMaximum(iDim);
      float   fRange = fMax - fMin;

      //if range == 0 we can not normalize and all values are the same -> no info for clustering
      if (fRange == 0) continue;

      //2.3. load the values of the current dimension of all clustercenters
      float[] pCenterValues = new float[m_iClusterNum];
      for (int iCluster = 0; iCluster < m_iClusterNum; iCluster++) {
        pCenterValues[iCluster] = m_pClusterCenters.getCenterValue(iCluster, iDim);
      }
      //System.out.println("2.3");

      //2.4. run through all entries
      for (int i = 0; i < iEntryNum; i++) {
        float fCurrval = pValues[i];
        fCurrval = (fCurrval - fMin)/fRange;

        //run through all cluster centers and add the distance of the current dimension to the overall distance
        for (int iCluster = 0; iCluster < m_iClusterNum; iCluster++) {
          float fCurrDistance = (fCurrval - pCenterValues[iCluster]);
          pDistanceTable[iCluster][i] += fDimensionWeight*fCurrDistance*fCurrDistance;
        }
      }
      //System.out.println("2.4.");


    }

    return pDistanceTable;


  }


  //calculate new cluster centers
  // but only the dimensions with weight > 0. The others can be calculated after the
  // clustering is finished
  private ClusterCenters calculateUsedDimensionsOfClusterCenters() {
    
    //run through all dimensions
    int iDimNum = m_pData.getDimNumber();
    ClusterCenters pNewCenters = new ClusterCenters(m_iClusterNum, iDimNum);

    for (int iDim = 0; iDim < iDimNum;iDim++) {

      //1. consider only dimensions with a weight > 0
      if (m_pDimensionWeights[iDim] <= 0) continue;

      //2. load the values of the current dimension
      float[] pValues = m_pData.getDimensionValues(iDim);
      float   fMin = m_pData.getDimensionMinimum(iDim);
      float   fMax = m_pData.getDimensionMaximum(iDim);
      float   fRange = fMax - fMin;

      if (fRange == 0) continue;



      //3. calculate the average dimension value for each cluster
      float[] pDimValues = new float[m_iClusterNum];
      for (int iCluster = 0; iCluster < m_iClusterNum; iCluster++) {
        pDimValues[iCluster] = 0.0f;
      }

      //4. sum up all the entry values
      int iEntryNum = m_pData.getEntryNum();
      for (int i = 0; i < iEntryNum; i++) {
        float fCurrValue = pValues[i];
        fCurrValue = (fCurrValue - fMin)/fRange;

        int iClusterId = (int)m_pClusterLabels[i];
        pDimValues[iClusterId] += fCurrValue;
      }

      //5. divide the sum of entries by their number and set the values in the cluster centers
      for (int iCluster = 0; iCluster < m_iClusterNum; iCluster++) {
        pDimValues[iCluster] /= (float)m_pClusterPopulations[iCluster];

        pNewCenters.setCenterValue(iCluster, iDim, pDimValues[iCluster]);
      }

    }

    return pNewCenters;

  }


  private void calculateNotUsedDimensionsOfClusterCenters() {

    int iDimNum = m_pData.getDimNumber();
    for (int iDim = 0; iDim < iDimNum; iDim++) {

      //1. consider only dimensions with a weight <= 0, because the other were already calculated
      if (m_pDimensionWeights[iDim] > 0) continue;

      //2. load the values of the current dimension
      float[] pValues = m_pData.getDimensionValues(iDim);
      float   fMin = m_pData.getDimensionMinimum(iDim);
      float   fMax = m_pData.getDimensionMaximum(iDim);
      float   fRange = fMax - fMin;
      if (fRange == 0) continue;

      //3. calculate the average dimension value for each cluster
      float[] pDimValues = new float[m_iClusterNum];
      for (int iCluster = 0; iCluster < m_iClusterNum; iCluster++) {
        pDimValues[iCluster] = 0.0f;
      }

      //4. sum up all the entry values
      int iEntryNum = m_pData.getEntryNum();
      for (int i = 0; i < iEntryNum; i++) {
        float fCurrValue = pValues[i];
        fCurrValue = (fCurrValue - fMin)/fRange;

        int iClusterId = (int)m_pClusterLabels[i];
        pDimValues[iClusterId] += fCurrValue;
      }

      //divide the sum of entries by their number and set the values in the cluster centers
      for (int iCluster = 0;iCluster < m_iClusterNum; iCluster++) {
        pDimValues[iCluster] /= (float)m_pClusterPopulations[iCluster];

        m_pClusterCenters.setCenterValue(iCluster, iDim, pDimValues[iCluster]);
      }

    }

  }


  //initiates calculation of the remaining dimensions for the cluster centers
  //saves the cluster labels and distances as datadimensions
  private void finishClustering() {

    //1. calculate the final cluster labels
    //1. a) calculate an array holding the distance of each data point to each cluster
    float[][] pDistanceTable = calculateDistanceTable();




    //1. b) calculate the label for each entry and its distance to its cluster center
    int iEntryNum = m_pData.getEntryNum();
    m_pClusterLabels = new float[iEntryNum];
    m_pClusterDistances = new float[iEntryNum];

    //run through all entries
    for (int i = 0; i < iEntryNum; i++) {

      //check which cluster center has the minimum distance
      float fMinDistance = pDistanceTable[0][i];
      int iBestClusterId = 0;

      for (int iCluster = 1; iCluster < m_iClusterNum; iCluster++) {
        float fCurrDistance = pDistanceTable[iCluster][i];
        if (fCurrDistance < fMinDistance) {
          fMinDistance = fCurrDistance;
          iBestClusterId = iCluster;
        }
      }

      m_pClusterLabels[i] = iBestClusterId;
      m_pClusterDistances[i] = fMinDistance;

      m_pClusterPopulations[iBestClusterId]++;
    }



    //2. calculate the remaining dimensions of the cluster centers (= dimensions that were not used for clustering)
    calculateNotUsedDimensionsOfClusterCenters();


    //3. save the clustering results
    //3. a) check if the clustering dimensions already exist (TO DO: Make decision to overwrite or write new cluster result)
    String[] pDimNames = m_pData.getDimensionNames();
    int iDimNum = pDimNames.length;
    boolean bLabels = false;
    boolean bDist = false;
    for (int i = 0; i < iDimNum; i++) {
      if (pDimNames[i].equals(CLUSTIDNAME)) {
        bLabels = true;
      }
      if (pDimNames[i].equals(CLUSTDISTNAME)) {
        bDist = true;
      }
    }


    //3. b) write the clustering results
    if (bLabels) {
      m_pData.setDimensionValues(CLUSTIDNAME, m_pClusterLabels);
    }
    else {
      DataDimension pClusterLabelDimension = new FloatDataDimension(m_pClusterLabels, CLUSTIDNAME);
      m_pData.addDimension(pClusterLabelDimension);
    }

    if (bDist) {
      m_pData.setDimensionValues(CLUSTDISTNAME, m_pClusterDistances);
    }
    else {
      DataDimension pClusterDistancesDimension = new FloatDataDimension(m_pClusterDistances, CLUSTDISTNAME);
      m_pData.addDimension(pClusterDistancesDimension);
    }

  }





}