package tools;

/*
 *  Copyright (C) 2018 Juergen Platzer 
 * 
 *  This file is part of JInfoViz.
 *  
 *  JInfoViz is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  JInfoViz is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *  
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with JInfoViz.  If not, see <http://www.gnu.org/licenses/>.
 */

import java.util.*;

public class PrincipalComponentAnalysis {


  private int       m_iDimNumber;
  private int       m_iEntryNumber;
  private float[][] m_pData;
  private float[][] m_pCovarianceMatrix;

  private Vector<ValueAndVector>    m_pPrincipalComponents; 
  //TO DO: consider to write a Principal component class instead of ValueAndVector
  private float     m_fSumOfVariances;

  private float[]   m_pUsedMeans;

  private float     m_fRobustness;                    //robustness parameter


  //constructor for classic pca
  public PrincipalComponentAnalysis(float[][] pData, float fRobustness) {

    //1. check the robustness factor
    if (fRobustness >= 1) fRobustness = 1.0f;
    if (fRobustness <= 0.5) fRobustness = 0.5f;
    m_fRobustness = fRobustness;

    //2. set the member variables (TO DO: check if pData == null)
    m_pData = pData;
    m_iDimNumber = pData.length;
    m_iEntryNumber = pData[0].length;

    //3. calculate the covariance Matrix
    float[][] m_pCovarianceMatrix;
    CovarianceCalculator pCovarianceCalculator = new CovarianceCalculator();

    if (m_fRobustness == 1) {
      m_pCovarianceMatrix = pCovarianceCalculator.classicCovarianceMatrix(pData);
    }
    else {
      m_pCovarianceMatrix = pCovarianceCalculator.robustCovarianceMatrix(m_fRobustness, pData);
    }

    //TODO: bring this back into the program with a new math library
    /*
    //4. calculate principal components
    Matrix pMatrix = new Matrix(m_pCovarianceMatrix);
    EigenvalueDecomposition pEigenvalueDecomposition = pMatrix.eig();

    double[] pEigenValues = pEigenvalueDecomposition.getRealEigenvalues();
    Matrix pEigenVectorMatrix = pEigenvalueDecomposition.getV();
    float[][] pEigenVectors = pEigenVectorMatrix.getArrayCopyFloat();
    */

    //5. fill the pricipal components vector with the eigenvalues and eigenvectors
    m_pPrincipalComponents = new Vector<ValueAndVector>();
    m_fSumOfVariances = 0.0f;
    
    for (int iCol = 0; iCol < m_iDimNumber; iCol++) {
      //TODO: bring this back into the program with a new math library
      //float fValue = (float)pEigenValues[iCol];
      float fValue = 0.0f;
      m_fSumOfVariances += fValue;

      float[] pVector = new float[m_iDimNumber];

      for (int iRow = 0; iRow < m_iDimNumber; iRow++) {
        //TODO: bring this back into the program with a new math library
        //pVector[iRow] = pEigenVectors[iRow][iCol];
        pVector[iRow] = 0.0f;
      }

      ValueAndVector pPrincipalComponent = new ValueAndVector(fValue, pVector);
      m_pPrincipalComponents.add(pPrincipalComponent);

    }

    //6. sort the principal components -> last principal component has the biggest eigenvalue = first princ.comp.
    Collections.sort(m_pPrincipalComponents);


    //7. used means = mean of data set
    m_pUsedMeans = new float[m_iDimNumber];
    for (int iDim = 0; iDim < m_iDimNumber; iDim++) {
      m_pUsedMeans[iDim] = calculateMean(pData[iDim]);
    }


  }


  /////////////////////////////////
  // GET METHODS                 //
  /////////////////////////////////

  //returns the explained variance by the components that are specified by an index array
  public float getExplainedVarianceByComponents(int[] pIndices) {
    //1. make some checks
    if (pIndices == null) return 0.0f;
    int iIndicesNum = pIndices.length;
    if (iIndicesNum == 0) return 0.0f;
    
    //2. calculate the value and return it
    float fExplainedVar = 0.0f;

    for (int i = 0; i < iIndicesNum; i++) {
      fExplainedVar += getExplainedVarianceByComponent(pIndices[i]);
    }

    return fExplainedVar;

  }


  //returns the explained variance by the component that is specified by index (1 = 1. principal component)
  public float getExplainedVarianceByComponent(int iIndex) {
    
    int iEntryID = m_pPrincipalComponents.size() - iIndex;

    if (iEntryID < 0) return 0.0f;
    if (iEntryID >= m_pPrincipalComponents.size()) return 0.0f;

    ValueAndVector pPrincipalComponent = m_pPrincipalComponents.get(iEntryID);

    return (pPrincipalComponent.getValue()/m_fSumOfVariances);

  }



  //returns for the given data the coordinates in the specified principal component
  //data = dim x entries
  public float[] getComponentCoordinates(int iComponentIndex, float[][] pData) {

    //1. make some checks
    if (pData == null) return null;
    if (iComponentIndex <= 0) return null;
    if (iComponentIndex > m_pPrincipalComponents.size()) return null;

    //2. get the needed principal component
    int iEntryId = m_pPrincipalComponents.size() - iComponentIndex;
    ValueAndVector pPrincipalComponent = m_pPrincipalComponents.get(iEntryId);
    float[] pPCValues = pPrincipalComponent.getVector();

    //3. extract dimensionality of data
    int iDimNum = pData.length;
    int iEntryNum = pData[0].length;

    //4. fill the component Coordinates array
    float[] pComponentCoordinates = new float[iEntryNum];

    for (int i = 0; i < iEntryNum; i++) {

      pComponentCoordinates[i] = 0.0f;

      for (int iDim = 0; iDim < iDimNum; iDim++) {
        pComponentCoordinates[i] += (pData[iDim][i] - m_pUsedMeans[iDim]) * pPCValues[iDim];
      }
    }

    return pComponentCoordinates;


  }



  //returns for data used for creation of the pca the coordinates in the specified principal component
  public float[] getComponentCoordinatesForData(int iComponentIndex) {

    //1. make some checks
    if (iComponentIndex <= 0) return null;
    if (iComponentIndex > m_pPrincipalComponents.size()) return null;

    //2. get the needed principal component
    int iEntryId = m_pPrincipalComponents.size() - iComponentIndex;
    ValueAndVector pPrincipalComponent = m_pPrincipalComponents.get(iEntryId);
    float[] pPCValues = pPrincipalComponent.getVector();

    //3. extract dimensionality of data
    int iDimNum = m_pData.length;
    int iEntryNum = m_pData[0].length;

    //fill the component Coordinates array
    float[] pComponentCoordinates = new float[iEntryNum];

    for (int i = 0; i < iEntryNum; i++) {

      pComponentCoordinates[i] = 0.0f;

      for (int iDim = 0; iDim < iDimNum; iDim++) {
        pComponentCoordinates[i] += (m_pData[iDim][i] - m_pUsedMeans[iDim]) * pPCValues[iDim];
      }
    }

    return pComponentCoordinates;


  }


  public ValueAndVector getPrincipalComponent(int iComponentIndex) {

    //make some checks
    if (iComponentIndex <= 0) return null;
    if (iComponentIndex > m_pPrincipalComponents.size()) return null;

    //get the needed principal component
    int iEntryId = m_pPrincipalComponents.size() - iComponentIndex;
    return m_pPrincipalComponents.get(iEntryId);


  }


  public float getRobustnessParameter() {
    return m_fRobustness;
  }

  public int getPrincipalComponentNumber() {
    return m_pPrincipalComponents.size();
  }



  // calculates the arithmetic mean of the given float array
  private float calculateMean(float[] pData) {
    //1. make some checks
    if (pData == null) return 0.0f;
    
    //2. do the calculation
    int iLength = pData.length;

    float fSum = 0.0f;
    for (int i = 0; i < iLength; i++) {
      fSum += pData[i];
    }
    return fSum/(float)iLength;

  }

}