package view;

/*
 *  Copyright (C) 2018 Juergen Platzer 
 * 
 *  This file is part of JInfoViz.
 *  
 *  JInfoViz is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  JInfoViz is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *  
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with JInfoViz.  If not, see <http://www.gnu.org/licenses/>.
 */

import java.util.*;
import java.awt.Color;

/** class implementing a color transition **/
public class ColorTransition {

  /*** member variables ***/
  private Vector<ColorPosition> m_pColorPositions;

  //////////////////////////////
  // CONSTRUCTORS             //
  //////////////////////////////
  /** constructor I **/
  public ColorTransition(ColorPosition[] pColorPositions) {

    //1. set the color positions
    m_pColorPositions = new Vector<ColorPosition>();

    //2. fill vector
    if (pColorPositions != null) {
      int iPosNum = pColorPositions.length;
      for (int i = 0; i < iPosNum; i++) {
        m_pColorPositions.add(pColorPositions[i]);
      }
    }

    //3. sort vector
    Collections.sort(m_pColorPositions);

    //4. set the borders for the colorpositions
    setColorPositionBorders();


  }


  /** constructor II **/
  public ColorTransition(ColorTransition pColorTransition) {

    //1. create vector
    m_pColorPositions = new Vector<ColorPosition>();

    //2. fill vector
    int iPosNum = pColorTransition.getColorPositionNumber();
    for (int i = 0; i < iPosNum; i++) {
      m_pColorPositions.add(new ColorPosition(pColorTransition.getColorPosition(i)));
    }

    //3. set the borders for the colorpositions
    setColorPositionBorders();



  }


  ///////////////////////////////////
  // GET METHODS                   //
  ///////////////////////////////////
  public ColorPosition getColorPosition(int iPosition) {
    if (iPosition < 0) iPosition = 0;
    if (iPosition >= m_pColorPositions.size()) iPosition = m_pColorPositions.size()-1;

    return (ColorPosition)m_pColorPositions.get(iPosition);

  }

  public int getColorPositionNumber() {
    return m_pColorPositions.size();
  }

  public Color getColorForPos(float fPosition) {
    //1. make some checks
    int iColorPosNum = m_pColorPositions.size();
    if (iColorPosNum == 0) return Color.BLACK;
    
    if (iColorPosNum == 1) return m_pColorPositions.get(0).getColor();
    
    //2. check the min and the max pos
    float fMinPosition = m_pColorPositions.get(0).getPosition();
    if (fPosition <= fMinPosition) return m_pColorPositions.get(0).getColor();
    
    float fMaxPosition = m_pColorPositions.get(iColorPosNum-1).getPosition();
    if (fPosition >= fMaxPosition) return m_pColorPositions.get(iColorPosNum-1).getColor();
    
    
  
    //3. calculate the upper and lower color position
    ColorPosition pLower = null;
    ColorPosition pHigher = null;

    //run through the list and calculate the neighbouring color positions
    for (int i = 0; i < iColorPosNum; i++) {
      ColorPosition pCurrColPos = m_pColorPositions.get(i);

      if (pCurrColPos.getPosition() < fPosition) {
        pLower = pCurrColPos;
      }
      else {
        pHigher = pCurrColPos;
      }

      //when we have a higher neighbour, we are done
      if (pHigher != null) break;

    }


    //4. calculate color mixture
    if ((pLower != null) && (pHigher != null)) {
      Color pColor1 = pLower.getColor();
      float fPosition1 = pLower.getPosition();
      Color pColor2 = pHigher.getColor();
      float fPosition2 = pHigher.getPosition();

      float fDistance = fPosition2 - fPosition1;
      float fFactor1 = 1 - ((fPosition - fPosition1)/fDistance);
      float fFactor2 = 1 - ((fPosition2 - fPosition)/fDistance);

      float fRed = fFactor1*((float)pColor1.getRed()/255.0f) + fFactor2*((float)pColor2.getRed()/255.0f);
      float fGreen = fFactor1*((float)pColor1.getGreen()/255.0f) + fFactor2*((float)pColor2.getGreen()/255.0f);
      float fBlue = fFactor1*((float)pColor1.getBlue()/255.0f) + fFactor2*((float)pColor2.getBlue()/255.0f);

      return new Color(fRed, fGreen, fBlue);


    }

    //5. return default values
    if (pLower == null) return pHigher.getColor();
    if (pHigher == null) return pLower.getColor();

    return Color.BLACK;

  }


  ///////////////////////////////////
  // ADD / DELETE METHODS          //
  ///////////////////////////////////
  public void deleteColorPosition(ColorPosition pColorPosition) {
    //1. make a simple check
    if (pColorPosition == null) return;
    
    //2. remove the color position, if we have only 2
    if (m_pColorPositions.size() > 2) {
      m_pColorPositions.remove(pColorPosition);
      setColorPositionBorders();
    }
  }

  public void addColorPosition(ColorPosition pColorPosition) {
    //1. make a simple check
    if (pColorPosition == null) return;
    
    //2. add the colo position and sort the color points
    m_pColorPositions.add(pColorPosition);
    Collections.sort(m_pColorPositions);
    setColorPositionBorders();

   
  }


  ///////////////////////////////////
  // PRIVATE METHODS               //
  ///////////////////////////////////

  /** sets the borders of all color positions, so that a position can only
      be changed between its neighbours **/
  private void setColorPositionBorders() {
    
    
    //1. set the neighbours for the interiour color points
    int iColPosNum = m_pColorPositions.size();

    for (int i = 1; i < iColPosNum-1; i++) {
      ColorPosition pCurrColPos = m_pColorPositions.get(i);
      ColorPosition pColPos0 = m_pColorPositions.get(i-1);
      ColorPosition pColPos1 = m_pColorPositions.get(i+1);

      pCurrColPos.setNeighbours(pColPos0, pColPos1);
    }

    //2. handle the first color position
    ColorPosition pCurrColPos = m_pColorPositions.get(0);
    ColorPosition pColPos1 = m_pColorPositions.get(1);
    pCurrColPos.setNeighbours(null, pColPos1);

    //3. last color position
    pCurrColPos = m_pColorPositions.get(iColPosNum-1);
    ColorPosition pColPos0 = m_pColorPositions.get(iColPosNum-2);
    pCurrColPos.setNeighbours(pColPos0, null);

  }


}



