package view;

/*
 *  Copyright (C) 2018 Juergen Platzer 
 * 
 *  This file is part of JInfoViz.
 *  
 *  JInfoViz is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  JInfoViz is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *  
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with JInfoViz.  If not, see <http://www.gnu.org/licenses/>.
 */

import listeners.IColorPanelListener;
import listeners.ITransitionPanelListener;
import listeners.IDataSetListener;
import listeners.IColoringListener;
import listeners.IViewColoring;
import listeners.ISelectionListListener;

import data.SelectionList;
import data.Data;

import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;
import javax.swing.*;
import javax.swing.event.*;
import java.io.File;
import java.util.Vector;

import javax.swing.border.TitledBorder;




//class handles coloring of the data items
public class ColoringFrame extends JFrame 
implements  WindowListener, ActionListener, ItemListener,
            IColorPanelListener, ITransitionPanelListener, IDataSetListener, ISelectionListListener {

  //member variables

  private IViewColoring       m_pCurrentColoring;     //object that manages the current coloring of the data in the views
  private IViewColoring       m_pLatestSelectionColoring;  //object that cashes the latest selection coloring
                                                //while another coloring mode is active
  private IViewColoring       m_pLatestDimensionColoring;  //object that cashes the latest dimension coloring
                                                //while another coloring mode is active

  /** JPanels that contain the GUI **/
  private JPanel m_pColorSelectionPanel, m_pColorDimensionPanel;

  /** ColorPanels holding the colors for the parallel coordinates view **/
  private ColorPanel m_pSelectedColorPanel, m_pNotSelectedColorPanel, m_pMinColorPanel, m_pMaxColorPanel;

  /** TransitionPanel showing the transition of the soft selections and the dimension coloring **/
  private TransitionPanel m_pSelectionTransitionPanel, m_pDimensionTransitionPanel;

  /** RadioButtons for showing the linear and the advanced transition **/
  private JRadioButton m_pSelectionNormTransRadioButton, m_pDimensionNormTransRadioButton;
  private JRadioButton m_pSelectionAdvancedTransRadioButton, m_pDimensionAdvancedTransRadioButton;

  /** RadioButtons indicating which Coloring is active **/
  private JRadioButton m_pSelectionColoringRadioButton, m_pDimensionColoringRadioButton;

  /** Choice holding all dimension names for coloring by dimension **/
  private Choice m_pDimensionChoice;

  /** Vector for the coloring listeners that should be notified if something changes here **/
  private Vector<IColoringListener> m_pColoringListeners;




  //Constructor, Creating of GUI
  public ColoringFrame(String[] pDimNames) {

    //1. set Window-Properties
    setResizable(true);
    enableEvents(AWTEvent.WINDOW_EVENT_MASK);
    addWindowListener(this);
    setSize(new Dimension(500, 350));
    setTitle("Color properties");

    //2. Containers-Properties and layout
    Container pContainer = getContentPane();
    BorderLayout pBorderLayout = new BorderLayout();
    pContainer.setLayout(pBorderLayout);

    //3. set member variables
    m_pCurrentColoring = new SelectionColoring(Color.BLACK, Color.RED, null);
    m_pLatestSelectionColoring = null;
    m_pLatestDimensionColoring = null;
    m_pColoringListeners = new Vector<IColoringListener>();



    //4. create the panels
    //color types panel
    JPanel pColorTypesPanel = createColorTypesPanel();

    //Panel for the Coloring by selection
    m_pColorSelectionPanel = createColoringBySelectionPanel();

    //Panel for the Coloring by dimension
    m_pColorDimensionPanel = createColoringByDimensionPanel(pDimNames);




    //5. Add Elements to Container
    pContainer.add(pColorTypesPanel, BorderLayout.NORTH);
    pContainer.add(m_pColorSelectionPanel, BorderLayout.CENTER);





  }

  ////////////////////////////////////
  // CREATING JPANELS               //
  ////////////////////////////////////

  /** creates the panel for the color types **/
  private JPanel createColorTypesPanel() {

    //1. create the panel and set the layout
    JPanel pColorTypesPanel = new JPanel();
    pColorTypesPanel.setLayout(new FlowLayout());

    //2. create the border
    TitledBorder pColorTypesBorder = new TitledBorder("Coloring types");
    pColorTypesPanel.setBorder(pColorTypesBorder);


    //3. coloring from selection
    m_pSelectionColoringRadioButton = new JRadioButton("By selection");
    m_pSelectionColoringRadioButton.setSelected(true);
    m_pSelectionColoringRadioButton.addActionListener(this);
    pColorTypesPanel.add(m_pSelectionColoringRadioButton);


    //4. coloring from dimension
    m_pDimensionColoringRadioButton = new JRadioButton("By dimension");
    m_pDimensionColoringRadioButton.addActionListener(this);
    pColorTypesPanel.add(m_pDimensionColoringRadioButton);


    //5. button group
    ButtonGroup pColoringTypesGroups = new ButtonGroup();
    pColoringTypesGroups.add(m_pSelectionColoringRadioButton);
    pColoringTypesGroups.add(m_pDimensionColoringRadioButton);
    
    return pColorTypesPanel;

  }

  /** creates the panel for the coloring from selection properties **/
  private JPanel createColoringBySelectionPanel() {

    //1. create panel and set layout
    JPanel pColorSelectionPanel = new JPanel();
    GridBagLayout pGridBagLayout = new GridBagLayout();
    pColorSelectionPanel.setLayout(pGridBagLayout);

    //2. set border
    TitledBorder pColorSelectionBorder = new TitledBorder("Coloring by selection");
    pColorSelectionPanel.setBorder(pColorSelectionBorder);


    //3. constraints for the layout of the components
    GridBagConstraints pConstraints = new GridBagConstraints();

    pConstraints.fill = GridBagConstraints.BOTH;
    pConstraints.weightx = 1.0;
    pConstraints.weighty = 0.1;
    pConstraints.insets = new Insets(5, 5, 5, 5);





    //4. not selected color
    JLabel pNotSelectedColorLabel = new JLabel("Color of not selected items: ");
    pGridBagLayout.setConstraints(pNotSelectedColorLabel, pConstraints);
    pColorSelectionPanel.add(pNotSelectedColorLabel);

    m_pNotSelectedColorPanel = new ColorPanel(Color.BLACK);
    m_pNotSelectedColorPanel.addColorPanelListener(this);
    pGridBagLayout.setConstraints(m_pNotSelectedColorPanel, pConstraints);
    pColorSelectionPanel.add(m_pNotSelectedColorPanel);


    //5. selected color
    JLabel pSelectedColorLabel = new JLabel("Color of selected items: ");
    pGridBagLayout.setConstraints(pSelectedColorLabel, pConstraints);
    pColorSelectionPanel.add(pSelectedColorLabel);

    m_pSelectedColorPanel = new ColorPanel(Color.RED);
    m_pSelectedColorPanel.addColorPanelListener(this);
    pConstraints.gridwidth = GridBagConstraints.REMAINDER;
    pGridBagLayout.setConstraints(m_pSelectedColorPanel, pConstraints);
    pColorSelectionPanel.add(m_pSelectedColorPanel);


    //6. soft selection transition
    JLabel pColorTransLabel = new JLabel("Color transition for soft selection:");
    pConstraints.insets = new Insets(15, 5, 5, 5);
    pConstraints.gridwidth = GridBagConstraints.REMAINDER;
    pGridBagLayout.setConstraints(pColorTransLabel, pConstraints);
    pColorSelectionPanel.add(pColorTransLabel);

    //7. normal color transition
    m_pSelectionNormTransRadioButton = new JRadioButton("Linear transition");
    m_pSelectionNormTransRadioButton.setSelected(true);
    m_pSelectionNormTransRadioButton.addActionListener(this);
    pConstraints.insets = new Insets(5, 5, 5, 5);
    pConstraints.gridwidth = 2;
    pGridBagLayout.setConstraints(m_pSelectionNormTransRadioButton, pConstraints);
    pColorSelectionPanel.add(m_pSelectionNormTransRadioButton);


    //8. advanced color transition
    m_pSelectionAdvancedTransRadioButton = new JRadioButton("Advanced transition");
    m_pSelectionAdvancedTransRadioButton.addActionListener(this);
    //pConstraints.gridwidth = 2;
    pConstraints.gridwidth = GridBagConstraints.REMAINDER;
    pGridBagLayout.setConstraints(m_pSelectionAdvancedTransRadioButton, pConstraints);
    pColorSelectionPanel.add(m_pSelectionAdvancedTransRadioButton);

    //9. button group
    ButtonGroup pTransitionTypes = new ButtonGroup();
    pTransitionTypes.add(m_pSelectionNormTransRadioButton);
    pTransitionTypes.add(m_pSelectionAdvancedTransRadioButton);


    //10. TransitionPanel
    m_pSelectionTransitionPanel = new TransitionPanel(Color.BLACK, Color.RED);
    m_pSelectionTransitionPanel.setText("0", "Degree Of Interest (DOI)", "1");
    m_pSelectionTransitionPanel.addTransitionPanelListener(this);
    pConstraints.weighty = 0.5;
    pConstraints.gridwidth = GridBagConstraints.REMAINDER;
    pGridBagLayout.setConstraints(m_pSelectionTransitionPanel, pConstraints);
    pColorSelectionPanel.add(m_pSelectionTransitionPanel);
    
    
    return pColorSelectionPanel;






  }


  /** creates the panel for the coloring by dimension properties **/
  private JPanel createColoringByDimensionPanel(String[] pDimNames) {
    //1. create panel and set layout
    JPanel pColorDimensionPanel = new JPanel();
    GridBagLayout pGridBagLayout = new GridBagLayout();
    pColorDimensionPanel.setLayout(pGridBagLayout);

    //2. set border
    TitledBorder pColorDimensionBorder = new TitledBorder("Coloring by dimension");
    pColorDimensionPanel.setBorder(pColorDimensionBorder);

    //3. constraints for the layout of the components
    GridBagConstraints pConstraints = new GridBagConstraints();

    pConstraints.fill = GridBagConstraints.BOTH;
    pConstraints.gridwidth = 2;
    pConstraints.weightx = 1.0;
    //pConstraints.weighty = 0.1;
    pConstraints.insets = new Insets(5, 5, 5, 5);



    //4. selection of the dimension
    JLabel pDimensionLabel = new JLabel("Dimensions: ");
    pConstraints.gridwidth = 2;
    pGridBagLayout.setConstraints(pDimensionLabel, pConstraints);
    pColorDimensionPanel.add(pDimensionLabel);

    m_pDimensionChoice = new Choice();
    if (pDimNames != null) {
      int iDimNum = pDimNames.length;
      for (int i = 0; i < iDimNum; i++) {
        m_pDimensionChoice.add(pDimNames[i]);
      }
    }
    m_pDimensionChoice.addItemListener(this);
    pConstraints.gridwidth = GridBagConstraints.REMAINDER;
    pGridBagLayout.setConstraints(m_pDimensionChoice, pConstraints);
    pColorDimensionPanel.add(m_pDimensionChoice);


    //5. minimum color
    JLabel pMinColorLabel = new JLabel("Color of minimum value: ");
    pConstraints.gridwidth = 2;
    pGridBagLayout.setConstraints(pMinColorLabel, pConstraints);
    pColorDimensionPanel.add(pMinColorLabel);

    m_pMinColorPanel = new ColorPanel(Color.BLUE);
    m_pMinColorPanel.addColorPanelListener(this);
    pGridBagLayout.setConstraints(m_pMinColorPanel, pConstraints);
    pColorDimensionPanel.add(m_pMinColorPanel);


    //6. maximum color
    JLabel pMaxColorLabel = new JLabel("Color of maximum value: ");
    pGridBagLayout.setConstraints(pMaxColorLabel, pConstraints);
    pColorDimensionPanel.add(pMaxColorLabel);

    m_pMaxColorPanel = new ColorPanel(Color.RED);
    m_pMaxColorPanel.addColorPanelListener(this);
    pConstraints.gridwidth = GridBagConstraints.REMAINDER;
    pGridBagLayout.setConstraints(m_pMaxColorPanel, pConstraints);
    pColorDimensionPanel.add(m_pMaxColorPanel);



    //7. soft selection transition
    JLabel pColorTransitionLabel = new JLabel("Color transition:");
    pConstraints.insets = new Insets(15, 5, 5, 5);
    pConstraints.gridwidth = GridBagConstraints.REMAINDER;
    pGridBagLayout.setConstraints(pColorTransitionLabel, pConstraints);
    pColorDimensionPanel.add(pColorTransitionLabel);

    //8. normal color transition
    m_pDimensionNormTransRadioButton = new JRadioButton("Linear transition");
    m_pDimensionNormTransRadioButton.setSelected(true);
    m_pDimensionNormTransRadioButton.addActionListener(this);
    pConstraints.insets = new Insets(5, 5, 5, 5);
    pConstraints.gridwidth = 2;
    pGridBagLayout.setConstraints(m_pDimensionNormTransRadioButton, pConstraints);
    pColorDimensionPanel.add(m_pDimensionNormTransRadioButton);


    //9. advanced color transition
    m_pDimensionAdvancedTransRadioButton = new JRadioButton("Advanced transition");
    m_pDimensionAdvancedTransRadioButton.addActionListener(this);
    //pConstraints.gridwidth = 2;
    pConstraints.gridwidth = GridBagConstraints.REMAINDER;
    pGridBagLayout.setConstraints(m_pDimensionAdvancedTransRadioButton, pConstraints);
    pColorDimensionPanel.add(m_pDimensionAdvancedTransRadioButton);

    //10. button group
    ButtonGroup pTransitionTypes = new ButtonGroup();
    pTransitionTypes.add(m_pDimensionNormTransRadioButton);
    pTransitionTypes.add(m_pDimensionAdvancedTransRadioButton);


    //11. add a color transition
    m_pDimensionTransitionPanel = new TransitionPanel(Color.BLUE, Color.RED);
    m_pDimensionTransitionPanel.setText("Min", "Dimension values", "Max");
    m_pDimensionTransitionPanel.addTransitionPanelListener(this);
    pConstraints.weighty = 0.5;
    pConstraints.gridwidth = GridBagConstraints.REMAINDER;
    pGridBagLayout.setConstraints(m_pDimensionTransitionPanel, pConstraints);
    pColorDimensionPanel.add(m_pDimensionTransitionPanel);

    return pColorDimensionPanel;
  }


  ////////////////////////////////////
  // ACTIONLISTENER                 //
  ////////////////////////////////////
  public void actionPerformed(ActionEvent e) {
    //System.out.println(e.getActionCommand());

    //1. radiobutton "by selection"
    if (e.getActionCommand().equals("By selection")) {
      //1.1. change the panels
      Container pContainer = getContentPane();
      pContainer.remove(m_pColorDimensionPanel);
      pContainer.add(m_pColorSelectionPanel, BorderLayout.CENTER);

      //1.2. invoke the correct painting
      m_pColorSelectionPanel.revalidate();
      m_pColorSelectionPanel.repaint();

      //1.3. create and set the new coloring for the views
      m_pLatestDimensionColoring = m_pCurrentColoring; //backup of the dimension coloring
      if (m_pLatestSelectionColoring == null) {
        m_pCurrentColoring = new SelectionColoring(m_pNotSelectedColorPanel.getColor(), m_pSelectedColorPanel.getColor(),
        m_pSelectionTransitionPanel.getAdvancedTransition());
      }
      else {
        m_pCurrentColoring = m_pLatestSelectionColoring;
      }
      colorPropertiesChanged();


      return;
    }

    //2. radiobutton "by dimension"
    if (e.getActionCommand().equals("By dimension")) {
      //2.1. change the panels
      Container pContainer = getContentPane();
      pContainer.remove(m_pColorSelectionPanel);
      pContainer.add(m_pColorDimensionPanel, BorderLayout.CENTER);

      //2.2. invoke the correct painting
      m_pColorDimensionPanel.revalidate();
      m_pColorDimensionPanel.repaint();

      //2.3. create and set the new coloring for the views
      m_pLatestSelectionColoring = m_pCurrentColoring; //backup of the selection coloring
      if (m_pLatestDimensionColoring == null) {
        m_pCurrentColoring = new DimensionColoring(m_pDimensionChoice.getSelectedIndex(), m_pMinColorPanel.getColor(), m_pMaxColorPanel.getColor(),
        m_pDimensionTransitionPanel.getAdvancedTransition());
      }
      else {
        m_pCurrentColoring = m_pLatestDimensionColoring;
      }
      colorPropertiesChanged();

      return;
    }

    //3. radiobutton "linear transition" was selected
    if (e.getActionCommand().equals("Linear transition")) {
      if (m_pSelectionNormTransRadioButton.isSelected()) {
        m_pSelectionTransitionPanel.setLinearTransition(m_pNotSelectedColorPanel.getColor(), m_pSelectedColorPanel.getColor());
        m_pSelectionTransitionPanel.setAdvancedTransitionVisible(false);
      }
      if (m_pDimensionNormTransRadioButton.isSelected()) {
       m_pDimensionTransitionPanel.setLinearTransition(m_pMinColorPanel.getColor(), m_pMaxColorPanel.getColor());
       m_pDimensionTransitionPanel.setAdvancedTransitionVisible(false);
      }

      transitionPanelChanged(null);
      return;
    }

    //4. radiobutton "advanced transition" was selected
    if (e.getActionCommand().equals("Advanced transition")) {
      if (m_pSelectionAdvancedTransRadioButton.isSelected()) {
          m_pSelectionTransitionPanel.setAdvancedTransitionVisible(true);
      }
      if (m_pDimensionAdvancedTransRadioButton.isSelected()) {
        m_pDimensionTransitionPanel.setAdvancedTransitionVisible(true);
      }

      transitionPanelChanged(null);

      return;
    }


  }

  ////////////////////////////////////
  // ITEMLISTENER                   //
  ////////////////////////////////////
  //the selection in the dimension choice has been changed. -> Set new coloring settings.
  public void itemStateChanged(ItemEvent e) {

    m_pCurrentColoring = new DimensionColoring(m_pDimensionChoice.getSelectedIndex(), 
    m_pMinColorPanel.getColor(), m_pMaxColorPanel.getColor(),
    m_pDimensionTransitionPanel.getAdvancedTransition());
    colorPropertiesChanged();


  }


  ////////////////////////////////////
  // WINDOWLISTENER                 //
  ////////////////////////////////////
  public void windowClosing(WindowEvent event1) {
    setVisible(false);
  }
  public void windowGainedFocus(WindowEvent event2) {}
  public void windowClosed(WindowEvent e) {}
  public void windowDeactivated(WindowEvent e) {}
  public void windowActivated(WindowEvent e) {}
  public void windowDeiconified(WindowEvent e) {}
  public void windowIconified(WindowEvent e) {}
  public void windowOpened(WindowEvent e) {}


  ////////////////////////////////////
  // COLOR PANEL LISTENER METHOD    //
  ////////////////////////////////////

  /** properties of a ColorPanel have changed
   -) update the endcolors of the transitionpanel
   -) update the coloring of the parcoords
   **/
  public void colorPanelChanged(ColorPanel pColorPanel) {

    //1. coloring by selection is active
    if (m_pSelectionColoringRadioButton.isSelected()) {

      m_pSelectionTransitionPanel.setLinearTransition(m_pNotSelectedColorPanel.getColor(), m_pSelectedColorPanel.getColor());

      m_pCurrentColoring = new SelectionColoring(m_pNotSelectedColorPanel.getColor(), m_pSelectedColorPanel.getColor(),
      m_pSelectionTransitionPanel.getAdvancedTransition());
      
      m_pSelectionTransitionPanel.revalidate();
      m_pSelectionTransitionPanel.repaint();
      
      
      colorPropertiesChanged();
      
    }

    //2. coloring by dimension is active
    else {
      m_pDimensionTransitionPanel.setLinearTransition(m_pMinColorPanel.getColor(), m_pMaxColorPanel.getColor());

      m_pCurrentColoring = new DimensionColoring(m_pDimensionChoice.getSelectedIndex(), 
      m_pMinColorPanel.getColor(), m_pMaxColorPanel.getColor(),
      m_pDimensionTransitionPanel.getAdvancedTransition());
      
      m_pDimensionTransitionPanel.revalidate();
      m_pDimensionTransitionPanel.repaint();
      
      colorPropertiesChanged();
      
      
    }

    //3. invoke the correct painting
    m_pColorSelectionPanel.revalidate();
    m_pColorSelectionPanel.repaint();



  }


  /////////////////////////////////////////
  // TRANSITION PANEL LISTENER METHOD    //
  /////////////////////////////////////////

  /** properties of a TransitionPanel have changed
    -) tell the coloring of the parcoord the changes
  **/
  public void transitionPanelChanged(TransitionPanel pTransistionPanel) {
    //1. coloring by selection is active
    if (m_pSelectionColoringRadioButton.isSelected()) {
      m_pCurrentColoring.setColorTransition(m_pSelectionTransitionPanel.getAdvancedTransition());
      colorPropertiesChanged();
    }
    //2. coloring by dimension is active
    else {
      m_pCurrentColoring.setColorTransition(m_pDimensionTransitionPanel.getAdvancedTransition());
      colorPropertiesChanged();
    }


  }


  ////////////////////////////////////
  // IDATASETLISTENER               //
  ////////////////////////////////////

  public void dimensionValuesChanged(Data pData, String pDimName) {
    //1. make a simple check 
    if (pDimName == null) return;
    
    //2. if coloring by dimension is active and the dimension is affected -> fire an event
    if (m_pDimensionColoringRadioButton.isSelected()) {
      if (m_pDimensionChoice.getSelectedItem().equals(pDimName)) {
        colorPropertiesChanged();
      }

    }
  }

  public void dimensionNumberChanged(Data pData, String pDimName) {
    //1. make a simple check
    if (pData == null) return;

    //2. update the choice of the dimension coloring
    //2.1. store the selected dimension and then remove all
    String pSelectedDimension = m_pDimensionChoice.getSelectedItem();
    m_pDimensionChoice.removeAll();

    //2.2. add all dimensions and set the selected
    String[] pDimensionNames = pData.getDimensionNames();
    int iDimNum = pDimensionNames.length;
    for (int i = 0; i < iDimNum; i++) {
      m_pDimensionChoice.add(pDimensionNames[i]);
      if (pDimensionNames[i].equals(pSelectedDimension)) {
        m_pDimensionChoice.select(i);
      }
    }

  }


  ////////////////////////////////////
  // ISELECTIONLISTLISTENER         //
  ////////////////////////////////////

  public void selectionListChanged(SelectionList pSelectionList) {
    //1. make a check
    if (pSelectionList == null) return;
  
    //2. set the selection list doi
    if (m_pSelectionColoringRadioButton.isSelected()) {
      m_pCurrentColoring.updateDOI(pSelectionList.getGlobalDOI());
      colorPropertiesChanged();
    }
  }



  //////////////////////////////////////////
  // COLORING LISTENER METHODS          //
  //////////////////////////////////////////
  public void addColoringListener(IColoringListener pListener) {
    if (pListener == null) return;
    
    m_pColoringListeners.add(pListener);
  }

  private void colorPropertiesChanged() {

    int iListenerNum = m_pColoringListeners.size();
    for (int i = 0; i < iListenerNum; i++) {
      IColoringListener pListener = m_pColoringListeners.get(i);
      pListener.coloringChanged(m_pCurrentColoring);
    }

  }




}