package view;

/*
 *  Copyright (C) 2018 Juergen Platzer 
 * 
 *  This file is part of JInfoViz.
 *  
 *  JInfoViz is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  JInfoViz is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *  
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with JInfoViz.  If not, see <http://www.gnu.org/licenses/>.
 */

import listeners.GlobalConstants;
import listeners.IAxisManagementListener;
import listeners.IExportable;
import listeners.IViewColoring;
import listeners.IColoringListener;

import data.AxisSelection;
import data.Data;
import data.SelectionList;
import data.Selection;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;
import java.util.*;

public class ParCoordView extends JPanel implements MouseListener, MouseMotionListener,
GlobalConstants, IAxisManagementListener, IExportable, IColoringListener, Runnable {



  /*** data specific variables ***/
  Data m_pData;          //data set


  /*** Drawing variables ***/
  int m_iDrawWidth;        //width within the view is drawn
  int m_iDrawHeight;       //height within the view is drawn
  float m_fAxisDistance;     //distance between the axis
  int[] m_pVisibleAxisOrder;        //array containing the visible axis in order of appearance (left to right)
                                    //maps the visible dimensions (0, 1, 2, ..., n) to their order in the dataset.
  boolean[] m_pFlippedAxis;         //array containing true if a axis should be flipped. only visible axis are considered.
                                    //oder is according their appearance in the dataset.

  BufferedImage m_pEmptyAxisImage;  //buffered image on which only the axis, dimension names and minima and maxima are drawn
  BufferedImage m_pDataImage;       //buffered image that contains the data items

  float[] m_pOffsets;        //for each dimension there is an offset for normalization
  float[] m_pScaleFactors;     //for each dimension there is a scalefactor for normalization
  boolean m_bRedrawAll;         //indicates if a redraw of the whole view should be made
  boolean m_bRedrawData;        //indicates if the data points have to be redrawn

  /*** COLORING ***/
  IViewColoring   m_pColoringView; //object managing all coloring operations

  /*** interaction ***/
  Interaction m_pInteraction;     //object handling the interaction functionality for this view

  /*** SelectionList ***/
  SelectionList m_pSelectionList;


  /*** settings for the drawing ***/
  private boolean m_bDrawDimNames;    //flag indicating, if the dimnames should be drawn
  private boolean m_bDrawDimMinMax;   //flag indicating, if the minima and maxima should be drawn


  /*** Thread for redrawing the data items ***/
  private Thread m_pRedrawDataThread;      //thread managing the redraw of the data points
  private boolean m_bRedrawDataDone;  //indicates if the redraw of the data has been done
  private java.util.Timer m_pTimer;   //initiates a new drawing of the data points


  ////////////////////////////////
  // CONSTRUCTOR                //
  ////////////////////////////////
  public ParCoordView(int iWidth, int iHeight, SelectionList pSelectionList) {
    //1. set size
    setSize(iWidth, iHeight);

    //2. add Mouse interface functionality
    addMouseListener(this);
    addMouseMotionListener(this);

    //3. set member variables
    
    //3.1. data, flags.
    m_pEmptyAxisImage = null;

    m_pData = null;
    m_bRedrawAll = false;
    m_bRedrawData = false;

    //3.2. coloring, interaction and selectionlist
    m_pColoringView = new SelectionColoring(Color.BLACK, Color.RED, null);

    m_pSelectionList = pSelectionList;
    m_pInteraction = new ParCoordSelectionInteraction(new Vector(), this, pSelectionList);


    //3.3. drawing variables
    m_bDrawDimNames = true;
    m_bDrawDimMinMax = true;

    //3.4. thread
    m_bRedrawDataDone = true;
    m_pTimer = null;


  }




  //calls the paint-method
  public void paintComponent(Graphics pGraphics) {
    super.paintComponent(pGraphics);
    update(pGraphics);
  }


  //paint-method
  public void update(Graphics pGraphics) {
    
    //1. init the graphics environment
    Graphics2D pGraphics2D = (Graphics2D)pGraphics;
    pGraphics2D.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
    pGraphics2D.setRenderingHint(RenderingHints.KEY_RENDERING, RenderingHints.VALUE_RENDER_SPEED);

    //2. skip if we have no data
    if (m_pData == null) return;

    //3. create a new backupImage of the view if the size has changed
    if ((m_pEmptyAxisImage == null) ||
        (m_pEmptyAxisImage.getWidth() != getWidth()) ||
        (m_pEmptyAxisImage.getHeight() != getHeight())) {

      m_pEmptyAxisImage = null;
      m_pDataImage = null;
      m_bRedrawAll = true;
    }


    //draw background
    if (m_bRedrawAll) {

      //calculate drawing variables
      calculateDrawingVariables();

      //draw the axis
      drawAxis();

    }



    //4. draw the data
    if (m_pDataImage == null) {
      drawData();
      m_bRedrawData = false;
      m_bRedrawAll = false;

    }

    
    //5. draw the final image
    pGraphics2D.drawImage(m_pDataImage, 0, 0, this);


    //6. draw selections
    drawAxisSelections(pGraphics2D);


    //7. start the thread for redrawing data, if necessary
    if (m_bRedrawAll || m_bRedrawData) {

      //start the thread for redrawing the data points if
      //the previous redraw thread has been done
      if (m_bRedrawDataDone) {
        m_pRedrawDataThread = new Thread(this);
        m_pRedrawDataThread.start();
        m_bRedrawDataDone = false;
      }
      else {
        if (m_pTimer != null) m_pTimer.cancel();
        m_pTimer = new java.util.Timer();
        m_pTimer.schedule(new RedrawTask(this), 200);
      }


    }

    m_bRedrawAll = false;
    m_bRedrawData = false;





  }




  ////////////////////////////////////////
  //DRAWING methods                     //
  ////////////////////////////////////////

  //calculates the main variables needed for the other drawing operations
  private void calculateDrawingVariables() {


    //1. calculate the drawing area
    int iWidth = getWidth();
    int iHeight = getHeight();
    m_iDrawWidth = iWidth - MARGIN_LEFT - MARGIN_RIGHT;
    m_iDrawHeight = iHeight - MARGIN_TOP - MARGIN_BOTTOM;

    //reduce m_iDrawHeight if dimension names or minima/maxima should be drawn
    if (m_bDrawDimNames) {
      m_iDrawHeight -= MARGIN_DIMNAMES;
    }
    if (m_bDrawDimMinMax) {
      m_iDrawHeight -= MARGIN_MINMAX;
    }


    //2. calculate the axis distance
    int iDimNum = m_pVisibleAxisOrder.length;
    m_fAxisDistance = (float)m_iDrawWidth/(float)(iDimNum-1);


    //3. create new backup image and draw the background
    m_pEmptyAxisImage = new BufferedImage(iWidth, iHeight, BufferedImage.TYPE_INT_RGB);
    Graphics2D pGraphics = m_pEmptyAxisImage.createGraphics();
    pGraphics.setColor(Color.WHITE);
    pGraphics.fillRect(0, 0, iWidth, iHeight);


    //4. create offset and a scale array
    m_pOffsets        = new float[iDimNum];
    m_pScaleFactors   = new float[iDimNum];

    for (int i = 0; i < iDimNum; i++) {
      float fMin    = m_pData.getDimensionMinimum(m_pVisibleAxisOrder[i]);
      float fMax    = m_pData.getDimensionMaximum(m_pVisibleAxisOrder[i]);

      m_pOffsets[i]        = fMin;
      m_pScaleFactors[i]   = fMax - fMin;

    }


  }

  //draws the axes of the parallel coordinates
  private void drawAxis() {
    //1. set up the graphics context
    Graphics2D pGraphics = m_pEmptyAxisImage.createGraphics();
    pGraphics.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
    pGraphics.setColor(Color.BLACK);

    //2. draw the axis
    int iDimNum = m_pVisibleAxisOrder.length;

    for (int i = 0; i < iDimNum; i++) {

      int iX1 = MARGIN_LEFT + (int)(i*m_fAxisDistance);
      int iX2 = iX1;

      int iY1 = MARGIN_TOP;
      int iY2 = MARGIN_TOP + m_iDrawHeight;

      if (m_bDrawDimMinMax) {
        iY1 += MARGIN_MINMAX/2;
        iY2 += MARGIN_MINMAX/2;
      }

      pGraphics.drawLine(iX1, iY1, iX2, iY2);

    }


    //3. draw dimnames
    if (m_bDrawDimNames) {

      //TO DO: Adapt fontsize and dimnames for small window size

      //3.1. set font
      pGraphics.setFont(new Font("Arial", Font.PLAIN, 12));
      FontMetrics pMetrics = pGraphics.getFontMetrics();

      //3.2. draw the dim names
      for (int i = 0; i < iDimNum; i++) {

        //fetch dimname
        String pCurrDimName = m_pData.getDimensionName(m_pVisibleAxisOrder[i]);

        //calculate position
        int iCurrStrWidth = pMetrics.stringWidth( pCurrDimName );
        int iCurrX = MARGIN_LEFT + (int)(i*m_fAxisDistance);
        int iCurrY = MARGIN_TOP + m_iDrawHeight + MARGIN_DIMNAMES;

        if (m_bDrawDimMinMax) {
          iCurrY += MARGIN_MINMAX;
        }

        pGraphics.drawString(pCurrDimName, iCurrX - iCurrStrWidth/2, iCurrY);
      }


    }


    //4. draw minmax
    if (m_bDrawDimMinMax) {
      //TO DO: Adapt fontsize and dimnames for small window size

      //4.1. set font
      pGraphics.setFont(new Font("Arial", Font.PLAIN, 12));
      FontMetrics pMetrics = pGraphics.getFontMetrics();

      //4.2. run through the dimensions
      for (int i = 0; i < iDimNum; i++) {

        //fetch dimension minimum and maximum
        String pCurrMin = Float.toString(m_pData.getDimensionMinimum(m_pVisibleAxisOrder[i]));
        String pCurrMax = Float.toString(m_pData.getDimensionMaximum(m_pVisibleAxisOrder[i]));

        //calculate position
        int iCurrMinWidth = pMetrics.stringWidth( pCurrMin );
        int iCurrMaxWidth = pMetrics.stringWidth( pCurrMax );

        int iCurrX = MARGIN_LEFT + (int)(i*m_fAxisDistance);
        int iCurrYMin;
        int iCurrYMax;

        if (m_pFlippedAxis[i]) {
          iCurrYMin = MARGIN_TOP;
          iCurrYMax = MARGIN_TOP + m_iDrawHeight + MARGIN_MINMAX;
        }
        //axis not flipped
        else {
          iCurrYMin = MARGIN_TOP + m_iDrawHeight + MARGIN_MINMAX;
          iCurrYMax = MARGIN_TOP;
        }


        pGraphics.drawString(pCurrMin, iCurrX - iCurrMinWidth/2, iCurrYMin);
        pGraphics.drawString(pCurrMax, iCurrX - iCurrMaxWidth/2, iCurrYMax);

      }


    }



  }


  //draws the data items
  private void drawData() {

    //1. create the image and the graphics context
    BufferedImage pImage = new BufferedImage(getWidth(), getHeight(), BufferedImage.TYPE_INT_RGB);
    Graphics2D pGraphics = pImage.createGraphics();
    pGraphics.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
    pGraphics.setRenderingHint(RenderingHints.KEY_RENDERING, RenderingHints.VALUE_RENDER_SPEED);

    //2. draw the background
    pGraphics.drawImage(m_pEmptyAxisImage, 0, 0, null);

    int[] pRenderPositions = m_pColoringView.getRenderPositions(m_pData);
    Color[] pRenderColors = m_pColoringView.getRenderColors(m_pData);

    int iDimNum = m_pVisibleAxisOrder.length;

    //fetch the values for the first dimension
    float[] pDim1Values = m_pData.getDimensionValues(m_pVisibleAxisOrder[0]);
    int[] pY1Positions = getPixelPositions(m_pVisibleAxisOrder[0], pDim1Values);
    
    for (int i = 0; i < iDimNum-1; i++) {

      //get dimension values of the next dimension
      float[] pDim2Values = m_pData.getDimensionValues(m_pVisibleAxisOrder[i+1]);
      if (pDim2Values == null) continue;

      //get entry num
      int iEntryNum = pDim1Values.length;
      if (iEntryNum != pDim2Values.length) continue;
      
      //calculate the x positions
      int iX1 = MARGIN_LEFT + (int)(i*m_fAxisDistance);
      int iX2 = MARGIN_LEFT + (int)((i+1)*m_fAxisDistance);
      
      //calculate the y positions
      int[] pY2Positions = getPixelPositions(m_pVisibleAxisOrder[i+1], pDim2Values);
      

      if (pRenderPositions == null) {

        for (int iEntry = 0; iEntry < iEntryNum; iEntry++) {

          pGraphics.setColor(pRenderColors[iEntry]);
          

          //int iY1 = getPixelPosition(m_pVisibleAxisOrder[i], pDim1Values[iEntry]);
          //int iY2 = getPixelPosition(m_pVisibleAxisOrder[i+1], pDim2Values[iEntry]);

          pGraphics.drawLine(iX1, pY1Positions[iEntry], iX2, pY2Positions[iEntry]);

          //if (iEntry%100 == 0) System.out.println(iEntry+" painted.");

        }

      }
      else {

        for (int iEntry = 0; iEntry < iEntryNum; iEntry++) {

          int iId = pRenderPositions[iEntry];
          pGraphics.setColor(pRenderColors[iEntry]);

          
          //int iY1 = getPixelPosition(m_pVisibleAxisOrder[i], pDim1Values[iId]);
          //int iY2 = getPixelPosition(m_pVisibleAxisOrder[i+1], pDim2Values[iId]);

          pGraphics.drawLine(iX1, pY1Positions[iId], iX2, pY2Positions[iId]);

          //if (iEntry%100 == 0) System.out.println(iEntry+" painted.");

        }
      }
      
      //set the next dimensions values now to the dim1 values
      pY1Positions = pY2Positions;

    }

    //set the image
    m_pDataImage = pImage;
    m_bRedrawDataDone = true;

  }


  //draws all axis selections
  private void drawAxisSelections(Graphics2D pGraphics) {

    //1. get the list of selections
    Vector<Selection> pSelectionList = m_pSelectionList.getLocalSelections(VIEW_ID_PAARCOORD);
    int iSelectionNum = pSelectionList.size();
    
    //2. set the stroke
    pGraphics.setStroke(new BasicStroke(2.0f));

    //3. process all selections
    for (int i = 0; i < iSelectionNum; i++) {

      //3.1. check if the selection has the correct type
      Selection pCurrSelection = pSelectionList.get(i);
      if (!(pCurrSelection instanceof AxisSelection)) continue;

      //3.2. check if the dimension of the selection is visible
      AxisSelection pCurrAxisSelection = (AxisSelection)pCurrSelection;
      int iDimID = pCurrAxisSelection.getDimId();
      int iVisibleDimID = calculateVisibleDimID(iDimID);
      if (iVisibleDimID == -1) continue;

      //3.3. calculate the x-coordinates
      int iXPos = MARGIN_LEFT + (int)(iVisibleDimID*m_fAxisDistance - m_fAxisDistance/8);
      int iSelWidth = (int)(m_fAxisDistance/4);

      //3.4. lower soft border
      if (pCurrAxisSelection.hasSoftLowerBorder()) {
        pGraphics.setColor(Color.BLUE);

        int iYPos = getPixelPosition(iDimID, pCurrAxisSelection.getLowerBorder());
        int iSelHeight = (int)(getPixelPosition(iDimID, pCurrAxisSelection.getSoftLowerBorder()) - iYPos);

        if (iSelHeight < 0) {
          iYPos += iSelHeight;
          iSelHeight = -iSelHeight;
        }


        pGraphics.drawRect(iXPos,iYPos,iSelWidth,iSelHeight);
      }

      //3.5. upper soft border
      if (pCurrAxisSelection.hasSoftUpperBorder()) {
        pGraphics.setColor(Color.BLUE);

        int iYPos = getPixelPosition(iDimID, pCurrAxisSelection.getSoftUpperBorder());
        int iSelHeight = (int)(getPixelPosition(iDimID, pCurrAxisSelection.getUpperBorder()) - iYPos);

        if (iSelHeight < 0) {
          iYPos += iSelHeight;
          iSelHeight = -iSelHeight;
        }


        pGraphics.drawRect(iXPos,iYPos,iSelWidth,iSelHeight);
      }


      //3.6. hard border
      pGraphics.setColor(Color.GREEN);
      int iYPos = getPixelPosition(iDimID, pCurrAxisSelection.getUpperBorder());
      int iSelHeight = (int)(getPixelPosition(iDimID, pCurrAxisSelection.getLowerBorder()) - iYPos);
      //System.out.println("x = " + iXPos + " y = " + y + " w = " + iSelWidth + " h = " + h);

      if (iSelHeight < 0) {
        iYPos += iSelHeight;
        iSelHeight = -iSelHeight;
      }
      pGraphics.drawRect(iXPos,iYPos,iSelWidth,iSelHeight);


    }
    
    //4. reset the stroke to the default value
    pGraphics.setStroke(new BasicStroke(1.0f));

  }




  ////////////////////////////////////////
  //Coordinate calculations             //
  ////////////////////////////////////////

  // calculates the y position for a certain value on the axes dimID in the dataset
  // if the given dimension is not visible Integer.MIN_VALUE is returned.
  public int getPixelPosition(int iDimID, float fValue) {

    int iVisibleDimID = calculateVisibleDimID(iDimID);
    if (iVisibleDimID == -1) return Integer.MIN_VALUE;

    float fNormValue = (fValue-m_pOffsets[iVisibleDimID])/m_pScaleFactors[iVisibleDimID];
    int iYPos;

    //if flipped
    if (m_pFlippedAxis[iVisibleDimID]) {
      iYPos = MARGIN_TOP + (int)((float)m_iDrawHeight * fNormValue);

      if (m_bDrawDimMinMax) {
        iYPos += MARGIN_MINMAX/2;
      }
    }
    //not flipped
    else {
      iYPos = getHeight() - MARGIN_BOTTOM - (int)((float)m_iDrawHeight * fNormValue);

      if (m_bDrawDimNames) {
        iYPos -= MARGIN_DIMNAMES;
      }
      if (m_bDrawDimMinMax) {
        iYPos -= MARGIN_MINMAX/2;
      }
    }

    return iYPos;
  }
  
  
  public int[] getPixelPositions(int iDimID, float[] pValues) {

    //1. make some checks
    if (pValues == null) return null;
    int iVisibleDimID = calculateVisibleDimID(iDimID);
    if (iVisibleDimID == -1) return null;
    
    //2. create the result the array
    int iEntryNum = pValues.length;
    int[] pPixelPositions = new int[iEntryNum];
    
    //3. make the y position calculations that are valid for every entry
    int iBasicYPos = 0;
    boolean bFlipped = m_pFlippedAxis[iVisibleDimID];
    float fOffset = m_pOffsets[iVisibleDimID];
    float fScaleFactor = m_pScaleFactors[iVisibleDimID];
    
    //if flipped
    if (bFlipped) {
      iBasicYPos = MARGIN_TOP;
      if (m_bDrawDimMinMax) {
        iBasicYPos += MARGIN_MINMAX/2;
      }
    }
    //if not flipped
    else {
      iBasicYPos = getHeight() - MARGIN_BOTTOM;

      if (m_bDrawDimNames) {
        iBasicYPos -= MARGIN_DIMNAMES;
      }
      if (m_bDrawDimMinMax) {
        iBasicYPos -= MARGIN_MINMAX/2;
      }
    }
    
    
    //4. run through all entries
    for (int i = 0; i < iEntryNum; i++) {
      float fNormValue = (pValues[i]-fOffset)/fScaleFactor;
      int iDrawingValue = (int)((float)m_iDrawHeight * fNormValue);
      
      if (bFlipped) {
        pPixelPositions[i] = iBasicYPos + iDrawingValue;
      }
      else {
        pPixelPositions[i] = iBasicYPos - iDrawingValue;
      }
      
    }
    

   
    return pPixelPositions;
  }

  // calculates the float value for a given y position on the axes dimID
  public float getCorrespondingValue(int iDimId, int iYPos) {

    int iVisibleDimID = calculateVisibleDimID(iDimId);
    if (iVisibleDimID == -1) return Float.NaN;

    float fNormValue;
     //if flipped
    if (m_pFlippedAxis[iVisibleDimID]) {
      iYPos -= MARGIN_TOP;
      if (m_bDrawDimMinMax) {
        iYPos -= MARGIN_MINMAX/2;
      }

      fNormValue = (float)(iYPos)/(float)m_iDrawHeight;
    }
    //not flipped
    else {
      iYPos -= getHeight();
      iYPos += MARGIN_BOTTOM;

      if (m_bDrawDimNames) {
        iYPos += MARGIN_DIMNAMES;
      }
      if (m_bDrawDimMinMax) {
        iYPos += MARGIN_MINMAX/2;
      }

      fNormValue = (-(float)iYPos)/(float)m_iDrawHeight;
    }

    float fValue = fNormValue*m_pScaleFactors[iVisibleDimID] + m_pOffsets[iVisibleDimID];
    return fValue;
  }


  //calculates for a certain x position the ID of a dimension in the dataset
  public int calculateDimID(int iXPos) {
    iXPos -= MARGIN_LEFT;
    int iVisibleDimID = (int)Math.rint((double)iXPos/m_fAxisDistance);

    if (iVisibleDimID < 0) return -1;
    if (iVisibleDimID >= m_pVisibleAxisOrder.length) return -1;

    return m_pVisibleAxisOrder[iVisibleDimID];
  }


  public float calculateCorrespondingDistanceValue(int iDimId, int iPixelDistance) {
    int iVisibleDimID = calculateVisibleDimID(iDimId);
    if (iVisibleDimID == -1) return Float.NaN;


    float fNormValue = -(float)iPixelDistance/(float)m_iDrawHeight;
    float fValue = fNormValue*m_pScaleFactors[iVisibleDimID];
    return fValue;
  }


  //returns the x coordinates of the left and right selection border
  public int[] getHorizontalSelectionBorders(int iDimId) {

    int iVisibleDimID = calculateVisibleDimID(iDimId);
    if (iVisibleDimID == -1) return null;


    int iXBorderLow = MARGIN_LEFT + (int)((float)iVisibleDimID*m_fAxisDistance - m_fAxisDistance/8);
    int iXBorderUp = MARGIN_LEFT + (int)((float)iVisibleDimID*m_fAxisDistance + m_fAxisDistance/8);

    int[] pBorders = {iXBorderLow, iXBorderUp};

    return pBorders;
  }


  //returns the visibility ID of a given dimension ID in the dataset (-1 if not visible is returned)
  private int calculateVisibleDimID(int iDimId) {
    int iVisibleAxisNum = m_pVisibleAxisOrder.length;
    for (int i = 0; i < iVisibleAxisNum; i++) {
      if (iDimId == m_pVisibleAxisOrder[i]) return i;
    }

    return -1;
  }






  ////////////////////////////////////////
  //MouseListener methods               //
  ////////////////////////////////////////
  //mouse clicked: show selection menu
  public void mouseClicked(MouseEvent e) {
    m_pInteraction.mouseClicked(e);
  }

  public void mouseEntered(MouseEvent e) {
    m_pInteraction.mouseEntered(e);
  }

  public void mouseExited(MouseEvent e) {
    m_pInteraction.mouseExited(e);
  }

  public void mousePressed(MouseEvent e) {
    m_pInteraction.mousePressed(e);
  }

  public void mouseReleased(MouseEvent e) {
    m_pInteraction.mouseReleased(e);
  }


  ////////////////////////////////////////
  //MouseMotionListener methods         //
  ////////////////////////////////////////
  public void mouseDragged(MouseEvent e) {
    m_pInteraction.mouseDragged(e);
  }

  public void mouseMoved(MouseEvent e) {
    m_pInteraction.mouseMoved(e);
  }





  //////////////////////////////////////////
  // interface for the key events         //
  // triggered from the mainframe         //
  //////////////////////////////////////////

  public void setControlPressed(boolean bPressed) {
    m_pInteraction.setControlPressed(bPressed);
  }



  //////////////////////////////////////////
  // GET METHODS                          //
  //////////////////////////////////////////



  public void renderImage(BufferedImage pImage) {
    //save old size
    int iOldWidth = getWidth();
    int iOldHeight = getHeight();

    //paint on the image
    Graphics2D pGraphics = pImage.createGraphics();
    int iWidth = pImage.getWidth();
    int iHeight = pImage.getHeight();
    setSize(iWidth, iHeight);
    m_bRedrawAll = true;
    paint(pGraphics);

    //reset the size
    setSize(iOldWidth, iOldHeight);


  }

  public float getAxisDistance() {
    return m_fAxisDistance;
  }

  //returns true if the dimension with the ID dimID in the dataset is visible and is flipped
  public boolean isAxisFlipped(int iDimId) {
    int iVisibleDimID = calculateVisibleDimID(iDimId);
    if (iVisibleDimID == -1) return false;

    return m_pFlippedAxis[iVisibleDimID];


  }


  //////////////////////////////////////////
  // SET METHODS                          //
  //////////////////////////////////////////

  //sets a new dataset
  public void setData(Data pData) {
    m_pData = pData;

    //visibility and flipped (all visible, no flipped axis)
    int iDimNum = m_pData.getDimNumber();
    m_pVisibleAxisOrder = new int[iDimNum];
    m_pFlippedAxis = new boolean[iDimNum];

    for (int i = 0; i < iDimNum; i++) {
      m_pVisibleAxisOrder[i] = i;
      m_pFlippedAxis[i] = false;
    }





    calculateDrawingVariables();
    //recalculateDOI(); //TO DO: handle this one

    //TO DO: delete all selections of the view
    //axisselectionlist = new Vector();

    m_pEmptyAxisImage = null;
    //currentAxisSelection = null;

    m_bRedrawAll = true;

    //set interaction to default
    repaint();
  }

  public void setInteraction(Interaction pInteraction) {
    m_pInteraction = pInteraction;
  }





  //////////////////////////////////////////
  // AXIS MANAGEMENT LISTENER INTERFACE   //
  //////////////////////////////////////////
  public void axisManagementChanged(AxisManagementFrame pManagement) {
    //1. make a simple check
    if (pManagement == null) return;
  
    //2. set the general axis properties
    boolean bDrawDimNames = pManagement.areDimensionNamesVisible();
    boolean bDrawDimMinMax = pManagement.areExtremaVisible();

    boolean bRedraw = false;

    if (bDrawDimNames != m_bDrawDimNames) {
      m_bDrawDimNames = bDrawDimNames;
      bRedraw = true;
    }

    if (bDrawDimMinMax != m_bDrawDimMinMax) {
      m_bDrawDimMinMax = bDrawDimMinMax;
      bRedraw = true;
    }


    //3. set the axis specific properties
    //3.1. visibility
    int[] pNewVisibleAxisOrder = pManagement.getVisibleAxisOrder();
    boolean bAxisOrderChanged = false;

    if (pNewVisibleAxisOrder.length != m_pVisibleAxisOrder.length) {
      bAxisOrderChanged = true;
    }
    else {
      int iVisibleAxisNum = m_pVisibleAxisOrder.length;
      for (int i = 0; i < iVisibleAxisNum; i++) {
        if (m_pVisibleAxisOrder[i] != pNewVisibleAxisOrder[i]) {
          bAxisOrderChanged = true;
          break;
        }
      }
    }

    if (bAxisOrderChanged) {
      m_pVisibleAxisOrder = pNewVisibleAxisOrder;
      bRedraw = true;
    }

    //3.2. flipped axis
    boolean[] pNewFlipAxis = pManagement.getFlippedAxis();
    boolean bFlipStatusChanged = false;

    if (m_pFlippedAxis.length != pNewFlipAxis.length) {
      bFlipStatusChanged = true;
    }
    else {
      int iFlippedAxisNum = m_pFlippedAxis.length;
      for (int i = 0; i < iFlippedAxisNum; i++) {
        if (m_pFlippedAxis[i] != pNewFlipAxis[i]) {
          bFlipStatusChanged = true;
          break;
        }
      }
    }

    if (bFlipStatusChanged) {
      m_pFlippedAxis = pNewFlipAxis;
      bRedraw = true;
    }



    //4. decide if we have to repaint all
    if (bRedraw) repaintAll();
  }






 //////////////////////////////////////////
  // CALLS FOR REPAINTING                 //
  //////////////////////////////////////////
  public void repaintSelections() {
    repaint();
  }

  public void repaintData() {
    m_bRedrawData = true;
    repaint();
  }


  //initializes a redraw of the view. redrawBackground indicates if all data items should be redrawn too.
  public void repaintAll() {
    m_bRedrawAll = true;
    repaint();
  }


  //////////////////////////////////////////
  // ICOLORINGLISTENER METHODS            //
  //////////////////////////////////////////

  /** Invoked when a change in the coloring of the data items occurs.
   *
   *  @param pColoringView IViewColoring object managing the colors of the data items.
   */
  public void coloringChanged(IViewColoring pColoringView) {
    m_pColoringView = pColoringView;

    repaintData();
  }



  //////////////////////////////////////////
  // RUNNABLE INTERFACE                   //
  //////////////////////////////////////////
  public void run() {
    drawData();

    Graphics2D pGraphics = (Graphics2D)getGraphics();

    pGraphics.drawImage(m_pDataImage, 0, 0, this);


    //draw selections
    drawAxisSelections(pGraphics);

  }
}