package view;

/*
 *  Copyright (C) 2018 Juergen Platzer 
 * 
 *  This file is part of JInfoViz.
 *  
 *  JInfoViz is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  JInfoViz is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *  
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with JInfoViz.  If not, see <http://www.gnu.org/licenses/>.
 */

import data.Data;
import listeners.IDataSetListener;
import gui.FileDialogHelpers;


import javax.swing.JFrame;
import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;
import javax.swing.*;
import javax.swing.event.*;
import java.io.File;
import javax.imageio.ImageIO;




/**
 * This class creates and handles the functionality the interface of the program.
 * It also represents the window in which the parallel coordinates view is shown and all functionalities
 * are called via the menus.
 *
 * @author    J&uuml;rgen Platzer
 */
//class creates and handles the functionality the interface of the application
public class ScatterplotFrame extends JFrame implements WindowListener, ActionListener, KeyListener, IDataSetListener{

  //member variables

  private ScatterplotView m_pScatterplotView;    //scatterplot view
  private Choice  m_pXDimChoice, m_pYDimChoice;             //choices for the selection of the dimensions for the scatterplot

  private JMenu m_pFileMenu;
  private JMenuItem m_pImageExportMenu;
  private ImageExportFrame      m_pImageExportFrame;       //jframe that handles the export of the images

  //Constructor, Creating of GUI

  /**
   * Constructor that creates the main window and objects of all other windows and views.
   * The needed listeners are registered.
   *
   */

  public ScatterplotFrame(ScatterplotView pScatterplotView, String[] pDimNames) {

    //1. set Window-Properties
    setResizable(true);
    enableEvents(AWTEvent.WINDOW_EVENT_MASK);
    setSize(new Dimension(805, 700));
    setTitle("2D Scatterplot");

    //2. add Listener functionality
    addWindowListener(this);
    addKeyListener(this);


    //3. set Containers-Properties
    Container pContainer = this.getContentPane();
    pContainer.setLayout(new BorderLayout());

    //4. set the member variables
    m_pScatterplotView = pScatterplotView;
    m_pImageExportFrame = new ImageExportFrame(m_pScatterplotView, this);

    //5. create the panel and add it to the frame
    JPanel pDimensionPanel = createDimensionPanel(pDimNames);
    pContainer.add(m_pScatterplotView, BorderLayout.CENTER);
    pContainer.add(pDimensionPanel, BorderLayout.SOUTH);


    //6. create menu, menu bar and set it in the frame
    m_pFileMenu = new JMenu("File");
    m_pImageExportMenu = new JMenuItem("Export image");
    m_pImageExportMenu.addActionListener(this);
    m_pFileMenu.add(m_pImageExportMenu);
    
    JMenuBar pMenubar = new JMenuBar();
    pMenubar.add(m_pFileMenu);
    setJMenuBar(pMenubar);

  }


  ////////////////////////////////////////
  //GUI CREATION METHODS                //
  ////////////////////////////////////////
  private JPanel createDimensionPanel(String[] pDimNames) {
    //1. create the panel and set the layout
    JPanel pDimensionPanel = new JPanel();
    pDimensionPanel.setLayout(new FlowLayout(FlowLayout.CENTER));


    //2. create the labels
    JLabel pXDimLabel = new JLabel("Dimension on X axis: ");
    JLabel pYDimLabel = new JLabel("Dimension on Y axis: ");

    //3. choices
    m_pXDimChoice = new Choice();
    m_pYDimChoice = new Choice();
    
    if (pDimNames != null) {
      int iDimNum = pDimNames.length;
      for (int i = 0; i < iDimNum; i++) {
        m_pXDimChoice.add(pDimNames[i]);
        m_pYDimChoice.add(pDimNames[i]);
      }
      
      if (iDimNum > 1) {
        m_pXDimChoice.select(0);
        m_pYDimChoice.select(1);
      }
    }

    m_pXDimChoice.addItemListener(new ItemListener() {
      public void itemStateChanged(ItemEvent e)  {
        xDim_ItemListener();
      }
    });
    m_pYDimChoice.addItemListener(new ItemListener() {
      public void itemStateChanged(ItemEvent e)  {
        yDim_ItemListener();
      }
    });




    //4. add the components to the panel
    pDimensionPanel.add(pXDimLabel);
    pDimensionPanel.add(m_pXDimChoice);
    pDimensionPanel.add(pYDimLabel);
    pDimensionPanel.add(m_pYDimChoice);

    return pDimensionPanel;

  }

  ////////////////////////////////////////
  //ITEMLISTENER METHODS                //
  ////////////////////////////////////////
  private void xDim_ItemListener() {
    m_pScatterplotView.setXAxis(m_pXDimChoice.getSelectedIndex());
  }

  private void yDim_ItemListener() {
    m_pScatterplotView.setYAxis(m_pYDimChoice.getSelectedIndex());
  }


  ////////////////////////////////////////
  //ActionListener method              //
  ////////////////////////////////////////

  /**
   * Handles the action that is initiated by choosing a MenuItem.
   *
   * @param e ActionEvent that identifies the MenuItem from which the event is triggered.
   */
  public void actionPerformed(ActionEvent e) {
    
    
    Object pSource = e.getSource();
    //1. image export
    if (pSource.equals(m_pImageExportMenu)) {


      //1.1. create and modify FileChooser
      JFileChooser pFileChooser = FileDialogHelpers.getImageFileChooser();
      File pSelectedFile = 
      FileDialogHelpers.showSaveFileDialog(pFileChooser, this);
      

      if (pSelectedFile != null) {      
        //open export frame
        m_pImageExportFrame.setVisible(true);
        m_pImageExportFrame.setOutputFile(pSelectedFile);
        m_pImageExportFrame.setExportable(m_pScatterplotView);
        setEnabled(false);

      }


      return;
    }



  }









  ////////////////////////////////////////
  //KeyListener methods                 //
  ////////////////////////////////////////

  /**
   * Sets a flag in the views to true if a certain key is pressed.
   */
   public void keyPressed(KeyEvent e) {
     if (e.getKeyCode() == KeyEvent.VK_CONTROL) {
       //parcoordview.setControlPressed(true);
     }
   }

  /**
   * Sets all flags indicating a pressed key in the views to false.
   */
   public void keyReleased(KeyEvent e) {
     //parcoordview.setControlPressed(false);
   }

  /**
   * Unused KeyListener method.
   */
   public void keyTyped(KeyEvent e) {
   }





  ////////////////////////////////////////
  //WindowListener methods              //
  ////////////////////////////////////////
  /**
   * Closing the window.
   */
  public void windowClosing(WindowEvent event1) {
    setVisible(false);
  }

  /**
   * Unused WindowListener method.
   */
  public void windowGainedFocus(WindowEvent event2) {}

  /**
   * Unused WindowListener method.
   */
  public void windowClosed(WindowEvent e) {}
  /**
   * Unused WindowListener method.
   */
  public void windowDeactivated(WindowEvent e) {}
  /**
   * Unused WindowListener method.
   */
  public void windowActivated(WindowEvent e) {}
  /**
   * Unused WindowListener method.
   */
  public void windowDeiconified(WindowEvent e) {}
  /**
   * Unused WindowListener method.
   */
  public void windowIconified(WindowEvent e) {}
  /**
   * Unused WindowListener method.
   */
  public void windowOpened(WindowEvent e) {}



  ////////////////////////////////////////
  //IDATASETLISTENER METHODS            //
  ////////////////////////////////////////
  //not used method
  public void dimensionValuesChanged(Data pData, String pDimensionName) {}

  //used to update the dimension choices
  public void dimensionNumberChanged(Data pData, String pDimensionName) {
    //1. make a simple check 
    if (pData == null) return;
    
    //2. get new dimension names
    String[] pDimNames = pData.getDimensionNames();
    int iDimNum = pDimNames.length;

    //3. save selected items
    String pSelectedXDim = m_pXDimChoice.getSelectedItem();
    String pSelectedYDim = m_pYDimChoice.getSelectedItem();

    //4. fill the choices with the new names
    m_pXDimChoice.removeAll();
    m_pYDimChoice.removeAll();
    for (int i = 0; i < iDimNum; i++) {
      m_pXDimChoice.add(pDimNames[i]);
      m_pYDimChoice.add(pDimNames[i]);
    }

    //set the previously selected items if possible
    m_pXDimChoice.select(pSelectedXDim);
    if (!(pSelectedXDim.equals(m_pXDimChoice.getSelectedItem()))) {
      m_pXDimChoice.select(0);
      xDim_ItemListener();
    }

    m_pYDimChoice.select(pSelectedYDim);
    if (!(pSelectedYDim.equals(m_pYDimChoice.getSelectedItem()))) {
      m_pYDimChoice.select(0);
      xDim_ItemListener();
    }


  }




}