package view;

/*
 *  Copyright (C) 2018 Juergen Platzer 
 * 
 *  This file is part of JInfoViz.
 *  
 *  JInfoViz is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  JInfoViz is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *  
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with JInfoViz.  If not, see <http://www.gnu.org/licenses/>.
 */


import data.ScatterplotSelection;
import data.SelectionList;
import data.Selection;
import listeners.GlobalConstants;
import listeners.IDataSubsetListener;

import javax.swing.*;
import java.awt.event.*;
import java.awt.Cursor;
import java.util.Vector;
import java.awt.geom.Point2D.Float;


/* class handling the interactive creation and manipulation of selections for the scatterplot view */
public class ScatterplotSelectionInteraction extends Interaction implements GlobalConstants, ActionListener {


  /** member variables **/
  private JPopupMenu        m_pAxisSelectionPopupMenu;       // popup menu for the AxisSelections
  private ScatterplotView   m_pScatterplotView;   // the scatterplot view

  /** MOUSE STATUS **/
  int m_iMouseStatus;       //is mouse in, out or on an horizontal border of a selection
  int m_iMouseDragStatus;   //should the mouse resize or reposition a selection or should a soft selection be created


  /** helpers for the selection manipulation **/
  float m_fXStart, m_fYStart;      //for the creation of the selection
                                    //indicating what the starting point of the selection is

  int m_iXDimID;                    //current id of the x axis
  int m_iYDimID;                    //current id of the y axis

  int m_iCorrX;                     //indicates to which axis the x dimension of the selection corresponds
  int m_iCorrY;                     //indicates to which axis the y dimension of the selection corresponds

  int     m_iRepositionX;           //needed for selection repositioning and resizing: holds the last x position
  int     m_iRepositionY;           //needed for selection repositioning and resizing: holds the last y position

  boolean m_bResizeUpperBorder;     //resizing: indicates if the upper border is resized
  boolean m_bResizeXBorder;         //resizing: indicates if the x dimension of the selection is modified
  boolean m_bResizeBothBorders;     //resizing: indicates if both dimensions of the selection are modified

  boolean m_bResizeXLowerBorder;    //corner resizing: indicates if the lower border of the x axis is modified
  boolean m_bResizeYLowerBorder;    //corner resizing: indicates if the lower border of the y axis is modified


  /**** key variables ****/
  boolean m_bCtrlPressed;       //indicates if control key is pressed (needed for drawing soft selection)

  /** selections **/
  ScatterplotSelection m_pCurrentSelection;   //scatterplot selection that is currently modified
  SelectionList m_pSelectionList;            //object managing the DOI and all selections



  //////////////////////////////
  // CONSTRUCTOR              //
  //////////////////////////////
  /** constructor **/
  public ScatterplotSelectionInteraction(
    Vector<IDataSubsetListener> pListeners, 
    ScatterplotView pScatterplotView, 
    SelectionList pSelectionList) 
  {
    //1. call the father constructor
    super(pListeners);

    //2. set member variables
    m_pScatterplotView = pScatterplotView;
    m_pSelectionList = pSelectionList;

    //3. create the popup menus
    createAxisSelectionPopupMenu();


    //4. set default mouse status
    m_iMouseStatus = MOUSE_OUT;
    m_iMouseDragStatus = MOUSE_DRAG_NONE;

    //5. set default helpers status
    m_fXStart = -1.0f;
    m_fYStart = -1.0f;
    m_iXDimID = -1;
    m_iYDimID = -1;


    //6. set default key status
    m_bCtrlPressed = false;


    //7. set default selection stauts
    m_pCurrentSelection = null;





  }



  ////////////////////////////////////////
  // MOUSE LISTENER METHODS             //
  ////////////////////////////////////////
  //mouse clicked: show selection menu
  public void mouseClicked(MouseEvent e) {

    //1. make a simple check -> only do something, when it is a right mouse click
    if (e.getButton() != MouseEvent.BUTTON3) return;

    //2. show the popup menu, when the mouse is within a selection or on the border of a selection
    if ((m_iMouseStatus == MOUSE_IN) || (m_iMouseStatus == MOUSE_HORIZONTAL))  {
      m_pAxisSelectionPopupMenu.show(m_pScatterplotView, e.getX(), e.getY());
    }

  }



  public void mouseEntered(MouseEvent e) {}
  public void mouseExited(MouseEvent e) {}


  //mouse button pressed
  public void mousePressed(MouseEvent e) {

    //1. do nothing if right mouse click was used or no current selection has been chosen
    if (e.getButton() == MouseEvent.BUTTON3) return;
    
    //2. create a new selection
    if ((m_iMouseStatus == MOUSE_OUT) && (m_pCurrentSelection == null)) {


      int iXPos = e.getX();
      int iYPos = e.getY();


      m_iXDimID = m_pScatterplotView.getXDimID();
      m_iYDimID = m_pScatterplotView.getYDimID();

      m_iMouseDragStatus = MOUSE_DRAG_CREATE;

      m_fXStart = m_pScatterplotView.getCorrespondingXValue(iXPos);
      m_fYStart = m_pScatterplotView.getCorrespondingYValue(iYPos);


      m_pCurrentSelection = new ScatterplotSelection(m_iXDimID, m_iYDimID, m_fXStart, m_fXStart, m_fYStart, m_fYStart,
      VIEW_ID_SCATTERPLOT);

      m_pSelectionList.addSelection(m_pCurrentSelection);

      return;

    }
    
    //3. if there is no current selection quit
    if (m_pCurrentSelection == null) return;

    

    //4. reposition the current selection
    if (m_iMouseStatus == MOUSE_IN) {
      m_iMouseDragStatus = MOUSE_DRAG_REPOSITION;

      m_iRepositionX = e.getX();
      m_iRepositionY = e.getY();

      calculateCorrespondence(m_pCurrentSelection.m_iXdimID, m_pCurrentSelection.m_iYdimID);

      return;
    }



    //5. change the horizontal border of the current selection or create a soft border
    if (m_iMouseStatus == MOUSE_HORIZONTAL) {

       m_iMouseDragStatus = MOUSE_DRAG_RESIZE_HOR;
       m_iRepositionY = e.getY();
       calculateCorrespondence(m_pCurrentSelection.m_iXdimID, m_pCurrentSelection.m_iYdimID);


       //check if both borders should be modified
       m_bResizeBothBorders = ((m_iCorrX == CORR_BOTH) || (m_iCorrY == CORR_BOTH));

       //check if x dimension of the selection should be modified
       if (m_bResizeBothBorders) {
         m_bResizeXBorder = (m_iCorrX == CORR_BOTH);
       }
       else {
         m_bResizeXBorder = (m_iCorrX == CORR_Y);
       }

       //check if the lower or upper border should be updated
       int iUpperDiff, iLowerDiff;
       if (m_bResizeXBorder) {
         iUpperDiff = m_iRepositionY - m_pScatterplotView.getYPixelPosition(m_pCurrentSelection.getUpperXBorder());
         iLowerDiff = m_iRepositionY - m_pScatterplotView.getYPixelPosition(m_pCurrentSelection.getLowerXBorder());
       }
       else {
         iUpperDiff = m_iRepositionY - m_pScatterplotView.getYPixelPosition(m_pCurrentSelection.getUpperYBorder());
         iLowerDiff = m_iRepositionY - m_pScatterplotView.getYPixelPosition(m_pCurrentSelection.getLowerYBorder());
       }


       m_bResizeUpperBorder = (Math.abs(iUpperDiff) < Math.abs(iLowerDiff));



       /*m_bResizeSoftBorder = false;
       int mindiff = Math.min(Math.abs(upperdiff), Math.abs(lowerdiff));

       if (m_currentAxisSelection.hasSoftUpperBorder()) {
         int softupperdiff = m_iRepositionY - m_pScatterplotView.getPixelPosition(dimID, m_currentAxisSelection.getSoftUpperBorder());

         if (Math.abs(softupperdiff) < mindiff) {
           mindiff = Math.abs(softupperdiff);
           m_bResizeUpperBorder = true;
           m_bResizeSoftBorder = true;
         }
       }
       if (m_currentAxisSelection.hasSoftLowerBorder()) {
         int softlowerdiff = m_iRepositionY - m_pScatterplotView.getPixelPosition(dimID, m_currentAxisSelection.getSoftLowerBorder());

         if (Math.abs(softlowerdiff) < mindiff) {
           mindiff = Math.abs(softlowerdiff);
           m_bResizeUpperBorder = false;
           m_bResizeSoftBorder = true;
         }
       }

       //if control is pressed and there is no soft border yet -> a soft border is created
       if (m_bResizeUpperBorder) {
         m_bCreateSoftBorder = m_bCtrlPressed && (!m_currentAxisSelection.hasSoftUpperBorder());
       }
       else {
         m_bCreateSoftBorder = m_bCtrlPressed && (!m_currentAxisSelection.hasSoftLowerBorder());
       }*/

       return;
    }


    //4. change the vertical border of the current selection or create a soft border
    if (m_iMouseStatus == MOUSE_VERTICAL) {

       m_iMouseDragStatus = MOUSE_DRAG_RESIZE_VER;
       m_iRepositionX = e.getX();
       calculateCorrespondence(m_pCurrentSelection.m_iXdimID, m_pCurrentSelection.m_iYdimID);


       //check if both borders should be modified
       m_bResizeBothBorders = ((m_iCorrX == CORR_BOTH) || (m_iCorrY == CORR_BOTH));

       //check if x dimension of the selection should be modified
       if (m_bResizeBothBorders) {
         m_bResizeXBorder = (m_iCorrX == CORR_BOTH);
       }
       else {
         m_bResizeXBorder = (m_iCorrX == CORR_X);
       }

       //check if the lower or upper border should be updated
       int iUpperDiff, iLowerDiff;
       if (m_bResizeXBorder) {
         iUpperDiff = m_iRepositionX - m_pScatterplotView.getXPixelPosition(m_pCurrentSelection.getUpperXBorder());
         iLowerDiff = m_iRepositionX - m_pScatterplotView.getXPixelPosition(m_pCurrentSelection.getLowerXBorder());
       }
       else {
         iUpperDiff = m_iRepositionX - m_pScatterplotView.getXPixelPosition(m_pCurrentSelection.getUpperYBorder());
         iLowerDiff = m_iRepositionX - m_pScatterplotView.getXPixelPosition(m_pCurrentSelection.getLowerYBorder());
       }


       m_bResizeUpperBorder = (Math.abs(iUpperDiff) < Math.abs(iLowerDiff));

       return;
    }


    /*//change the horizontal and vertical border of the current selection by dragging the north west corner
    //on x axis the minimum is modified, on y axis the maximum is modified
    if ((m_iMouseStatus == MOUSE_CORNER_NW) && (m_pCurrentSelection != null)) {

       m_iMouseDragStatus = MOUSE_DRAG_RESIZE_CORNER;

       m_iRepositionX = e.getX();
       m_iRepositionY = e.getY();

       calculateCorrespondence(m_pCurrentSelection.m_iXdimID, m_pCurrentSelection.m_iYdimID);

       //check if both borders should be modified
       m_bResizeBothBorders = ((m_iCorrX == CORR_BOTH) || (m_iCorrY == CORR_BOTH));

       if (m_bResizeBothBorders) {
         m_bResizeXLowerBorder = (m_iCorrX == CORR_BOTH);
         m_bResizeYLowerBorder = (m_iCorrY == CORR_BOTH);
       }
       else {
         if (m_iCorrX == CORR_X) {
           m_bResizeXLowerBorder = true;
         }
         else if (m_iCorrX == CORR_Y) {
           m_bResizeXLowerBorder = false;
         }

         if (m_iCorrY == CORR_X) {
           m_bResizeYLowerBorder = true;
         }
         else if (m_iCorrY == CORR_Y) {
           m_bResizeYLowerBorder = false;
         }

       }

       return;
    }*/


    
  }



  //mouse is released: update all selection settings and initialize a recalculation of the view
  public void mouseReleased(MouseEvent e) {
    if (m_iMouseDragStatus != MOUSE_DRAG_NONE) {
      mouseDragged(e);
      m_iMouseDragStatus = MOUSE_DRAG_NONE;
      m_pScatterplotView.repaintSelections();
    }
  }


  ////////////////////////////////////////
  // MOUSE MOTION LISTENER METHODS      //
  ////////////////////////////////////////
  public void mouseDragged(MouseEvent e) {
  
    //1. make an initial check -> if there is no current selection quit
    if (m_pCurrentSelection == null) return;

    //2. repositioning of a selection
    if (m_iMouseDragStatus == MOUSE_DRAG_REPOSITION) {

      //2.1. get the mouse position
      int iXPos = e.getX();
      int iYPos = e.getY();

      //2.2. calculate the value difference in the data 
      float fXDiff = m_pScatterplotView.calculateCorrespondingXDistanceValue(iXPos - m_iRepositionX);
      float fYDiff = m_pScatterplotView.calculateCorrespondingYDistanceValue(iYPos - m_iRepositionY);

      //2.3. only the x borders should be effected
      if (m_iCorrX == CORR_BOTH) {
        m_pCurrentSelection.setUpperXBorder(fXDiff + m_pCurrentSelection.getUpperXBorder());
        m_pCurrentSelection.setLowerXBorder(fXDiff + m_pCurrentSelection.getLowerXBorder());
      }

      //2.4. only the y borders should be effected
      else if (m_iCorrY == CORR_BOTH) {
        m_pCurrentSelection.setUpperYBorder(fYDiff + m_pCurrentSelection.getUpperYBorder());
        m_pCurrentSelection.setLowerYBorder(fYDiff + m_pCurrentSelection.getLowerYBorder());
      }


      //2.5. normal handling of the axes
      else {
        if (m_iCorrX == CORR_X) {
          m_pCurrentSelection.setUpperXBorder(fXDiff + m_pCurrentSelection.getUpperXBorder());
          m_pCurrentSelection.setLowerXBorder(fXDiff + m_pCurrentSelection.getLowerXBorder());
        }
        else if (m_iCorrX == CORR_Y) {
          m_pCurrentSelection.setUpperXBorder(fYDiff + m_pCurrentSelection.getUpperXBorder());
          m_pCurrentSelection.setLowerXBorder(fYDiff + m_pCurrentSelection.getLowerXBorder());
        }

        if (m_iCorrY == CORR_X) {
          m_pCurrentSelection.setUpperYBorder(fXDiff + m_pCurrentSelection.getUpperYBorder());
          m_pCurrentSelection.setLowerYBorder(fXDiff + m_pCurrentSelection.getLowerYBorder());
        }
        else if (m_iCorrY == CORR_Y) {
          m_pCurrentSelection.setUpperYBorder(fYDiff + m_pCurrentSelection.getUpperYBorder());
          m_pCurrentSelection.setLowerYBorder(fYDiff + m_pCurrentSelection.getLowerYBorder());
        }
      }


      /*if (m_currentAxisSelection.hasSoftUpperBorder()) {
        m_currentAxisSelection.setSoftUpperBorder(valuedifference + m_currentAxisSelection.getSoftUpperBorder());
      }
      if (m_currentAxisSelection.hasSoftLowerBorder()) {
        m_currentAxisSelection.setSoftLowerBorder(valuedifference + m_currentAxisSelection.getSoftLowerBorder());
      }*/

      m_iRepositionX = iXPos;
      m_iRepositionY = iYPos;


      m_pScatterplotView.repaintSelections();


      return;

    }


    //3. change the border of the current selection or create a soft border
    if (m_iMouseDragStatus == MOUSE_DRAG_RESIZE_HOR) {

      int iYPos = e.getY();
      float fValueDifference = m_pScatterplotView.calculateCorrespondingYDistanceValue(iYPos - m_iRepositionY);
      boolean bIsValueSet;

      if (m_bResizeXBorder) {
        if (m_bResizeUpperBorder) {
          bIsValueSet = m_pCurrentSelection.setUpperXBorder(fValueDifference + m_pCurrentSelection.getUpperXBorder());
        }
        else {
          bIsValueSet = m_pCurrentSelection.setLowerXBorder(fValueDifference + m_pCurrentSelection.getLowerXBorder());
        }
      }
      else {
        if (m_bResizeUpperBorder) {
          bIsValueSet = m_pCurrentSelection.setUpperYBorder(fValueDifference + m_pCurrentSelection.getUpperYBorder());
        }
        else {
          bIsValueSet = m_pCurrentSelection.setLowerYBorder(fValueDifference + m_pCurrentSelection.getLowerYBorder());
        }
      }

      //only if the value has been set a new m_iRepositionY is stored!
      if (bIsValueSet) m_iRepositionY = iYPos;
      m_pScatterplotView.repaintSelections();


      return;

    }

    //4. change the border of the current selection or create a soft border
    if (m_iMouseDragStatus == MOUSE_DRAG_RESIZE_VER) {

      int iXPos = e.getX();
      float fValueDifference = m_pScatterplotView.calculateCorrespondingXDistanceValue(iXPos - m_iRepositionX);
      boolean bIsValueSet;

      if (m_bResizeXBorder) {

        if (m_bResizeUpperBorder) {
          bIsValueSet = m_pCurrentSelection.setUpperXBorder(fValueDifference + m_pCurrentSelection.getUpperXBorder());
        }
        else {
          bIsValueSet = m_pCurrentSelection.setLowerXBorder(fValueDifference + m_pCurrentSelection.getLowerXBorder());
        }
      }
      else {
        if (m_bResizeUpperBorder) {
          bIsValueSet = m_pCurrentSelection.setUpperYBorder(fValueDifference + m_pCurrentSelection.getUpperYBorder());
        }
        else {
          bIsValueSet = m_pCurrentSelection.setLowerYBorder(fValueDifference + m_pCurrentSelection.getLowerYBorder());
        }

      }

      //only if the value has been set a new m_iRepositionY is stored!
      if (bIsValueSet) m_iRepositionX = iXPos;
      m_pScatterplotView.repaintSelections();

      return;

    }


    //change the border of the current selection by dragging the corner
    /*if ((m_iMouseDragStatus == MOUSE_DRAG_RESIZE_CORNER) && (m_pCurrentSelection != null)) {

      int xpos = e.getX();
      int ypos = e.getY();


      boolean isXvalueset, isYvalueset;


      float xDiff = m_pScatterplotView.calculateCorrespondingXDistanceValue(xpos - m_iRepositionX);
      float yDiff = m_pScatterplotView.calculateCorrespondingXDistanceValue(ypos - m_iRepositionY);

      if (m_bResizeBothBorders) {
        return;

      }
      else {

        if (m_bResizeXLowerBorder) {
          isXvalueset = m_pCurrentSelection.setLowerXBorder(xDiff + m_pCurrentSelection.getLowerXBorder());
        }
        else {
          isXvalueset = m_pCurrentSelection.setUpperXBorder(xDiff + m_pCurrentSelection.getUpperXBorder());
        }

        if (m_bResizeYLowerBorder) {
          isYvalueset = m_pCurrentSelection.setLowerYBorder(yDiff + m_pCurrentSelection.getLowerYBorder());
        }
        else {
          isYvalueset = m_pCurrentSelection.setUpperYBorder(yDiff + m_pCurrentSelection.getUpperYBorder());
        }
      }


      //only if the value has been set a new m_iRepositionY is stored!
      if (isXvalueset) m_iRepositionX = xpos;
      if (isYvalueset) m_iRepositionY = ypos;

      m_pScatterplotView.repaintSelections();

      return;
    }*/



    //5. creating a new selection
    if (m_iMouseDragStatus == MOUSE_DRAG_CREATE) {
      int iXPos = e.getX();
      int iYPos = e.getY();

      float fXEnd = m_pScatterplotView.getCorrespondingXValue(iXPos);
      float fYEnd = m_pScatterplotView.getCorrespondingYValue(iYPos);

      m_pSelectionList.deleteSelection(m_pCurrentSelection, false);

      m_pCurrentSelection = new ScatterplotSelection(m_iXDimID, m_iYDimID, m_fXStart, fXEnd, m_fYStart, fYEnd,
      VIEW_ID_SCATTERPLOT);

      m_pSelectionList.addSelection(m_pCurrentSelection);

      m_pScatterplotView.repaintSelections();
      return;

    }

  }

  public void mouseMoved(MouseEvent e) {
    //1. make an initial check
    if (m_iMouseDragStatus != MOUSE_DRAG_NONE) return;

    //2. recalculate the mouse status
    recalculateMouseStatus(e.getX(), e.getY());

    //3. set the cursor according to the mouse status
    switch (m_iMouseStatus) {
      case MOUSE_OUT: {
        m_pScatterplotView.setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
        break;
      }
      case MOUSE_HORIZONTAL: {
        m_pScatterplotView.setCursor(new Cursor(Cursor.N_RESIZE_CURSOR ));
        break;
      }
      case MOUSE_VERTICAL: {
        m_pScatterplotView.setCursor(new Cursor(Cursor.E_RESIZE_CURSOR ));
        break;
      }
      case MOUSE_IN: {
        m_pScatterplotView.setCursor(new Cursor(Cursor.MOVE_CURSOR));
        break;
      }
      case MOUSE_CORNER_NE: {
        m_pScatterplotView.setCursor(new Cursor(Cursor.NE_RESIZE_CURSOR));
        break;
      }
      case MOUSE_CORNER_NW: {
        m_pScatterplotView.setCursor(new Cursor(Cursor.NW_RESIZE_CURSOR));
        break;
      }
      case MOUSE_CORNER_SE: {
        m_pScatterplotView.setCursor(new Cursor(Cursor.SE_RESIZE_CURSOR));
        break;
      }
      case MOUSE_CORNER_SW: {
        m_pScatterplotView.setCursor(new Cursor(Cursor.SW_RESIZE_CURSOR));
        break;
      }

    }


  }


  //////////////////////////////////////////
  // interface for the key events         //
  // triggered from the mainframe         //
  //////////////////////////////////////////
  public void setControlPressed(boolean bPressed) {
    m_bCtrlPressed = bPressed;
  }




  // calculates the status of the a mouse position and sets the current AxisSelection
  // 0 = outside of AxisSelections
  // 1 = on horizontal border of a selection
  // 2 = in the selection
  public void recalculateMouseStatus(int iXPos, int iYPos) {

    //1. get variables of the scatterplot
    m_iXDimID = m_pScatterplotView.getXDimID();
    m_iYDimID = m_pScatterplotView.getYDimID();
    int iDrawingWidth = m_pScatterplotView.getDrawingWidth();
    int iDrawingHeight = m_pScatterplotView.getDrawingHeight();

    //2. check if x is in drawing area
    if (iXPos < ARROW_SPACE) return;
    if (iXPos > ARROW_SPACE + iDrawingWidth) return;

    //3. check if y is in drawing area
    if (iYPos < ARROW_TOP_LENGTH) return;
    if (iYPos > ARROW_TOP_LENGTH + iDrawingHeight) return;


    //4. run through the selections and check the mouse state
    Vector<Selection> pSelections = m_pSelectionList.getLocalSelections(VIEW_ID_SCATTERPLOT);
    int iSelectionNum = pSelections.size();

    for (int i = 0; i < iSelectionNum; i++) {

      //4.1. get the current selection properties
      ScatterplotSelection pCurrSelection = (ScatterplotSelection)pSelections.get(i);
      int iSelectionXDim = pCurrSelection.m_iXdimID;
      int iSelectionYDim = pCurrSelection.m_iYdimID;

      //4.2. skip if no correspondence
      calculateCorrespondence(iSelectionXDim, iSelectionYDim);
      if ((m_iCorrX == CORR_NO) && (m_iCorrY == CORR_NO)) continue;


      //4.3. calculate iXMin and iXMax
      int iXMin, iXMax;
      if (m_iCorrX == CORR_X) {
        iXMin = m_pScatterplotView.getXPixelPosition(pCurrSelection.getLowerXBorder());
        iXMax = m_pScatterplotView.getXPixelPosition(pCurrSelection.getUpperXBorder());
      }
      else if (m_iCorrY == CORR_X) {
        iXMin = m_pScatterplotView.getXPixelPosition(pCurrSelection.getLowerYBorder());
        iXMax = m_pScatterplotView.getXPixelPosition(pCurrSelection.getUpperYBorder());
      }
      else {
        iXMin = Integer.MIN_VALUE;
        iXMax = Integer.MAX_VALUE;
      }



      //4.4. calculate yMin and iYMax
      int iYMin, iYMax;
      if (m_iCorrX == CORR_Y) {
        iYMin = m_pScatterplotView.getYPixelPosition(pCurrSelection.getLowerXBorder());
        iYMax = m_pScatterplotView.getYPixelPosition(pCurrSelection.getUpperXBorder());
      }
      else if (m_iCorrY == CORR_Y) {
        iYMin = m_pScatterplotView.getYPixelPosition(pCurrSelection.getLowerYBorder());
        iYMax = m_pScatterplotView.getYPixelPosition(pCurrSelection.getUpperYBorder());
      }
      else {
        iYMin = Integer.MIN_VALUE;
        iYMax = Integer.MAX_VALUE;
      }



      //4.5. special cases:
      if (m_iCorrX == CORR_BOTH) {
        iXMin = m_pScatterplotView.getXPixelPosition(pCurrSelection.getLowerXBorder());
        iXMax = m_pScatterplotView.getXPixelPosition(pCurrSelection.getUpperXBorder());
        iYMin = m_pScatterplotView.getYPixelPosition(pCurrSelection.getLowerXBorder());
        iYMax = m_pScatterplotView.getYPixelPosition(pCurrSelection.getUpperXBorder());
      }

      if (m_iCorrY == CORR_BOTH) {
        iXMin = m_pScatterplotView.getXPixelPosition(pCurrSelection.getLowerYBorder());
        iXMax = m_pScatterplotView.getXPixelPosition(pCurrSelection.getUpperYBorder());
        iYMin = m_pScatterplotView.getYPixelPosition(pCurrSelection.getLowerYBorder());
        iYMax = m_pScatterplotView.getYPixelPosition(pCurrSelection.getUpperYBorder());
      }

      //4.6. make sure that iXMin is smaller than iXMax
      if (iXMin > iXMax) {
        int iSwap = iXMin;
        iXMin = iXMax;
        iXMax = iSwap;
      }


      //4.7. make sure that iXMin is smaller than iXMax
      if (iYMin > iYMax) {
        int iSwap = iYMin;
        iYMin = iYMax;
        iYMax = iSwap;
      }

      //4.8. mouse in the selection
      if ((iXPos >= iXMin + 5) && (iXPos <= iXMax - 5) && (iYPos >= iYMin + 5) && (iYPos <= iYMax - 5)) {
        m_iMouseStatus = MOUSE_IN;
        m_pCurrentSelection = pCurrSelection;
        return;
      }

      //4.9. mouse on horizontal border
      if ((iXPos >= iXMin + 5) && (iXPos <= iXMax - 5)) {

        //lower border
        if ((iYPos >= iYMin - 5) && (iYPos <= iYMin + 5)) {
          m_iMouseStatus = MOUSE_HORIZONTAL;
          m_pCurrentSelection = pCurrSelection;
          return;
        }

        //upper border
        if ((iYPos >= iYMax - 5) && (iYPos <= iYMax + 5)) {
          m_iMouseStatus = MOUSE_HORIZONTAL;
          m_pCurrentSelection = pCurrSelection;
          return;
        }
      }


      //4.10. mouse on vertical border
      if ((iYPos >= iYMin + 5) && (iYPos <= iYMax - 5)) {

        //lower border
        if ((iXPos >= iXMin - 5) && (iXPos <= iXMin + 5)) {
          m_iMouseStatus = MOUSE_VERTICAL;
          m_pCurrentSelection = pCurrSelection;
          return;
        }

        //upper border
        if ((iXPos >= iXMax - 5) && (iXPos <= iXMax + 5)) {
          m_iMouseStatus = MOUSE_VERTICAL;
          m_pCurrentSelection = pCurrSelection;
          return;
        }
      }


      /*//mouse on a left corner of the selection
      if ((xPos >= iXMin - 5) && (xPos <= iXMin + 5)) {

        //left upper corner
        if ((yPos >= iYMin - 5) && (yPos <= iYMin + 5)) {
          m_iMouseStatus = MOUSE_CORNER_NW;
          m_pCurrentSelection = currSel;
          return;
        }

        //left lower corner
        if ((yPos >= iYMax - 5) && (yPos <= iYMax + 5)) {
          m_iMouseStatus = MOUSE_CORNER_SW;
          m_pCurrentSelection = currSel;
          return;
        }
      }


      //mouse on a right corner of the selection
      if ((xPos >= iXMax - 5) && (xPos <= iXMax + 5)) {

        //right upper corner
        if ((yPos >= iYMin - 5) && (yPos <= iYMin + 5)) {
          m_iMouseStatus = MOUSE_CORNER_NE;
          m_pCurrentSelection = currSel;
          return;
        }

        //right lower corner
        if ((yPos >= iYMax - 5) && (yPos <= iYMax + 5)) {
          m_iMouseStatus = MOUSE_CORNER_SE;
          m_pCurrentSelection = currSel;
          return;
        }
      }*/



    }



    //5. if no selection was found set the default status
    m_iMouseStatus = MOUSE_OUT;
    m_pCurrentSelection = null;

  }



  //calculates to which axis of the scatterplot the dimensions of the selection correspond
  private void calculateCorrespondence(int iSelectionXDimId, int iSelectionYDimId) {

    //1. check the correspondence for the X dimension of the selection
    if (iSelectionXDimId == m_iXDimID) {
      if (iSelectionXDimId == m_iYDimID) {
        m_iCorrX = CORR_BOTH;
      }
      else {
        m_iCorrX = CORR_X;
      }
    }
    else if (iSelectionXDimId == m_iYDimID) {
      m_iCorrX = CORR_Y;
    }
    else {
      m_iCorrX = CORR_NO;
    }

    //2. check the correspondence for the Y dimension of the selection
    if (iSelectionYDimId == m_iXDimID) {
      if (iSelectionYDimId == m_iYDimID) {
        m_iCorrY = CORR_BOTH;
      }
      else {
        m_iCorrY = CORR_X;
      }
    }
    else if (iSelectionYDimId == m_iYDimID) {
      m_iCorrY = CORR_Y;
    }
    else {
      m_iCorrY = CORR_NO;
    }
  }




  ////////////////////////////////////////
  //POPUP MENUS                         //
  ////////////////////////////////////////
  private void createAxisSelectionPopupMenu() {

    m_pAxisSelectionPopupMenu = new JPopupMenu();

    JMenuItem pDeleteSelection = new JMenuItem("Delete selection");
    pDeleteSelection.addActionListener(this);
    m_pAxisSelectionPopupMenu.add(pDeleteSelection);

    JMenuItem pDeleteAllSelections = new JMenuItem("Delete all selections");
    pDeleteAllSelections.addActionListener(this);
    m_pAxisSelectionPopupMenu.add(pDeleteAllSelections);

    JMenuItem pSetSelectedDataAsData = new JMenuItem("Work with selected data");
    pSetSelectedDataAsData.addActionListener(this);
    m_pAxisSelectionPopupMenu.add(pSetSelectedDataAsData);

    JMenuItem pResetWholeDatasetAsData = new JMenuItem("Work with whole dataset");
    pResetWholeDatasetAsData.addActionListener(this);
    m_pAxisSelectionPopupMenu.add(pResetWholeDatasetAsData);

  }


  //////////////////////////////////////////
  //ActionListeners: for popup menu items //
  //////////////////////////////////////////
  public void actionPerformed(ActionEvent e) {

    //1. delete selection menu
    if (e.getActionCommand().equals("Delete selection")) {
      if (m_pCurrentSelection != null) {
        m_pSelectionList.deleteSelection(m_pCurrentSelection, true);
        m_pCurrentSelection = null;
        m_pScatterplotView.repaintSelections();
      }
      return;
    }

    //2. delete all selections menu
    if (e.getActionCommand().equals("Delete all selections")) {
      m_pSelectionList.deleteAllLocalSelections(VIEW_ID_SCATTERPLOT);
      m_pCurrentSelection = null;
      m_pScatterplotView.repaintSelections();
      return;
    }

    //3. work with selected data
    if (e.getActionCommand().equals("Work with selected data")) {
      setSelectedDataAsData();
      return;
    }

    //4. set whole dataset as data
    if (e.getActionCommand().equals("Work with whole dataset")) {
      resetWholeDatasetAsData();
      return;
    }

  }



}