package view;

/*
 *  Copyright (C) 2018 Juergen Platzer 
 * 
 *  This file is part of JInfoViz.
 *  
 *  JInfoViz is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  JInfoViz is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *  
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with JInfoViz.  If not, see <http://www.gnu.org/licenses/>.
 */


import listeners.GlobalConstants;
import listeners.IAxisManagementListener;
import listeners.IExportable;
import listeners.IColoringListener;
import listeners.IViewColoring;

import data.ScatterplotSelection;
import data.Data;
import data.Selection;
import data.SelectionList;

import javax.swing.*;
import java.awt.*;

import java.awt.event.*;
import java.awt.image.*;
import java.util.*;

public class ScatterplotView extends JPanel implements MouseListener, MouseMotionListener,
GlobalConstants, IExportable, IColoringListener, Runnable {



  /*** data specific variables ***/
  Data m_pData;          //data set


  /*** Images for displaying the view ***/
  private BufferedImage m_pEmptyAxisImage;  //contains the axis and the white background
  private BufferedImage m_pDataImage;       //contains the axis, background and data points

  /*** Flags indicating what is to redraw ***/
  private boolean m_bRedrawAll;             //everything has to be redrawn
  private boolean m_bRedrawData;           //the data has to be redrawn
  private boolean m_bRedrawSelections;     //only the selection borders are redrawn


  /*** data variables ***/
  private int     m_iXDimID;    //id of the dimension that is on the x axis
  private int     m_iYDimID;    //id of the dimension that is on the y axis
  private float   m_fXMin;      //minimum data value on the x axis
  private float   m_fXSpan;     //span of data values on the x axis
  private float   m_fXMax;      //maximum data value on the x axis
  private float   m_fYMin;      //minimum data value on the y axis
  private float   m_fYSpan;     //span of data values on the y axis
  private float   m_fYMax;      //maximum data value on the y axis
  private String  m_pXDimName;  //name of the dimension on the x axis
  private String  m_pYDimName;  //name of the dimension on the y axis

  /*** drawing variables ***/
  private int     m_iDrawingWidth;  //width in pixels for drawing data points
  private int     m_iDrawingHeight; //height in pixels for drawing data points
  private int     m_iWidth;         //width of the view
  private int     m_iHeight;        //height of the view

  /*** COLORING ***/
  IViewColoring   m_pColoring; //object managing coloring of data points

  /*** interaction ***/
  Interaction m_pInteraction;     //object handling the interaction functionality for this view

  /*** SelectionList ***/
  SelectionList m_pSelectionList;


  /*** Thread for redrawing the data items ***/
  Thread m_pRedrawDataThread;      //thread managing the redraw of the data points
  boolean m_bRedrawDataDone;  //indicates if the redraw of the data has been done
  private java.util.Timer m_pTimer;   //initiates a new drawing of the data points




  //constructor
  public ScatterplotView(int iWidth, int iHeight, SelectionList pSelectionList) {
    
    //1. set basic panel settings
    setSize(iWidth, iHeight);

    //2. add Mouse interface functionality
    addMouseListener(this);
    addMouseMotionListener(this);

    //3. set member variables (background image)
    m_pEmptyAxisImage = null;
    m_pData = null;

    //coloring, interaction and selectionlist
    m_pColoring = new SelectionColoring(Color.BLACK, Color.RED, null);

    m_pSelectionList = pSelectionList;
    m_pInteraction = new ScatterplotSelectionInteraction(new Vector(), this, pSelectionList);


    //thread
    m_bRedrawDataDone = true;



  }




  //calls the paint-method
  public void paintComponent(Graphics g) {
    super.paintComponent(g);
    update(g);
  }


  //paint-method
  public void update(Graphics g) {
    //1. make some checks
    if (m_pData == null) return;
    
    //2. get the graphics context and configure it.
    Graphics2D pGraphics = (Graphics2D)g;
    pGraphics.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
    pGraphics.setRenderingHint(RenderingHints.KEY_RENDERING, RenderingHints.VALUE_RENDER_SPEED);

    


    //3. check if size has changed and make redraw settings accordingly
    if ((m_pEmptyAxisImage == null) ||
        (m_pEmptyAxisImage.getWidth() != getWidth()) ||
        (m_pEmptyAxisImage.getHeight() != getHeight())) {

      m_bRedrawAll = true;
      m_pDataImage = null;
    }


    //4. redraw the background image if necessary
    if (m_bRedrawAll) {
      redrawBackgroundImage();
    }

    //5. redraw the data image if necessary
    if (m_pDataImage == null) {
      redrawDataImage();
      m_bRedrawData = false;
      m_bRedrawAll = false;
    }


    //6. draw the finished image and the selections
    pGraphics.drawImage(m_pDataImage, 0, 0, this);
    redrawSelections(pGraphics);


    //7. start the thread for redrawing data, if necessary
    if (m_bRedrawAll || m_bRedrawData) {
      //start the thread for redrawing the data points if
      //the previous redraw thread has been done
      if (m_bRedrawDataDone) {
        m_pRedrawDataThread = new Thread(this);
        m_pRedrawDataThread.start();
        m_bRedrawDataDone = false;
      }
      else {
        if (m_pTimer != null) m_pTimer.cancel();
        m_pTimer = new java.util.Timer();
        m_pTimer.schedule(new RedrawTask(this), 200);
      }

    }


    //8. set the redraw flags
    m_bRedrawAll = false;
    m_bRedrawData = false;


  }




  ////////////////////////////////////////
  //DRAWING methods                     //
  ////////////////////////////////////////

  //redraws the image m_pEmptyAxisImage that contains the axis and the white background
  private void redrawBackgroundImage() {
    
    //1. create the background image and fetch the graphics context
    int iWidth = getWidth();
    int iHeight = getHeight();

    BufferedImage pImage = new BufferedImage(iWidth, iHeight, BufferedImage.TYPE_INT_RGB);
    Graphics2D pGraphics = pImage.createGraphics();
    pGraphics.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
    pGraphics.setRenderingHint(RenderingHints.KEY_RENDERING, RenderingHints.VALUE_RENDER_SPEED);

    //2. draw backgound
    pGraphics.setColor(Color.WHITE);
    pGraphics.fillRect(0, 0, iWidth, iHeight);

    //3. draw x axis arrow
    //3.1. x-axis:rectangle
    pGraphics.setColor(new Color(0, 155, 0));

    int iXStart = (ARROW_SPACE - ARROW_THICKNESS)/2;
    int iYStart = iHeight - (ARROW_SPACE - ARROW_THICKNESS)/2 - ARROW_THICKNESS;
    int iArrowWidth = iWidth - iXStart - ARROW_TOP_LENGTH;
    pGraphics.fillRect(iXStart, iYStart, iArrowWidth, ARROW_THICKNESS);

    //3.2. x-axis: dimension name
    pGraphics.setFont(new Font("Arial", Font.BOLD, 15));
    FontMetrics pFontMetrics = pGraphics.getFontMetrics();
    int iStringLength = pFontMetrics.stringWidth(m_pXDimName);

    int iStringStart = (iArrowWidth - iStringLength)/2 + (ARROW_SPACE - ARROW_THICKNESS)/2;
    pGraphics.drawString(m_pXDimName, iStringStart, iHeight-5);

    //3.3. x-axis: minimum
    float fMin = (float)((int)(m_fXMin*10))/10;
    pGraphics.drawString(fMin+"", ARROW_SPACE, iHeight-5);

    //3.4. x-axis: maximum
    float fMax = (float)((int)(m_fXMax*10))/10;
    iStringStart = iWidth - ARROW_TOP_LENGTH - pFontMetrics.stringWidth(fMax+"");
    pGraphics.drawString(fMax+"", iStringStart, iHeight-5);



    //3.5. x-axis: arrow top
    Polygon pArrowTop = new Polygon();
    int iXCoord = iXStart + iArrowWidth;
    int iYCoord = iHeight - (ARROW_SPACE - ARROW_TOP_WIDTH)/2;
    pArrowTop.addPoint(iXCoord,iYCoord);
    iYCoord = iHeight - (ARROW_SPACE - ARROW_TOP_WIDTH)/2 - ARROW_TOP_WIDTH;
    pArrowTop.addPoint(iXCoord,iYCoord);
    iXCoord = iWidth-1;
    iYCoord = iHeight - ARROW_SPACE/2;
    pArrowTop.addPoint(iXCoord,iYCoord);

    pGraphics.fill(pArrowTop);





    //4. y axis arrow
    //4.1. y-axis:rectangle
    pGraphics.setColor(new Color(0, 0, 155));

    iXStart = (ARROW_SPACE - ARROW_THICKNESS)/2;
    iYStart = ARROW_TOP_LENGTH;
    int iArrowHeight = iHeight - (ARROW_SPACE - ARROW_THICKNESS)/2 - ARROW_TOP_LENGTH;
    pGraphics.fillRect(iXStart, iYStart, ARROW_THICKNESS, iArrowHeight);

    //4.2. y-axis: arrow top
    pArrowTop = new Polygon();
    iXCoord = (ARROW_SPACE - ARROW_TOP_WIDTH)/2;
    iYCoord = ARROW_TOP_LENGTH;
    pArrowTop.addPoint(iXCoord,iYCoord);
    iXCoord = (ARROW_SPACE - ARROW_TOP_WIDTH)/2 + ARROW_TOP_WIDTH;
    pArrowTop.addPoint(iXCoord,iYCoord);
    iXCoord = ARROW_SPACE/2;
    iYCoord = 0;
    pArrowTop.addPoint(iXCoord,iYCoord);

    pGraphics.fill(pArrowTop);

    //4.3. y-axis: dimension name
    pGraphics.rotate(Math.PI/2);
    iStringLength = pFontMetrics.stringWidth(m_pYDimName);
    iStringStart = ARROW_TOP_LENGTH + (iArrowHeight-iStringLength)/2;

    pGraphics.drawString(m_pYDimName, iStringStart, -5);


    //4.4. y-axis: maximum
    fMax = (float)((int)(m_fYMax*10))/10;
    pGraphics.drawString(fMax+"", ARROW_TOP_LENGTH, -5);

    //4.5. y-axis: minimum
    fMin = (float)((int)(m_fYMin*10))/10;
    iStringStart = iHeight - ARROW_SPACE - pFontMetrics.stringWidth(fMin+"");
    pGraphics.drawString(fMin+"", iStringStart, -5);

    
    
    //5. set m_pEmptyAxisImage
    m_pEmptyAxisImage = pImage;

  }


  //redraws the image m_pDataImage that is drawn on basis of m_pEmptyAxisImage
  //here the data points are drawn
  private void redrawDataImage() {

    //1. calculate drawing variables
    m_iWidth = getWidth();
    m_iDrawingWidth = m_iWidth - ARROW_SPACE - ARROW_TOP_LENGTH;
    m_iHeight = getHeight();
    m_iDrawingHeight = m_iHeight - ARROW_SPACE - ARROW_TOP_LENGTH;


    //2. get the data values
    float[] pXValues = m_pData.getDimensionValues(m_iXDimID);
    if (pXValues == null) return;
    
    float[] pYValues = m_pData.getDimensionValues(m_iYDimID);
    if (pYValues == null) return;

    int iEntryNum = pXValues.length;

    //3. create a new data image and get the graphics context
    BufferedImage pImage = new BufferedImage(m_iWidth, m_iHeight, BufferedImage.TYPE_INT_RGB);
    Graphics2D pGraphics = pImage.createGraphics();
    pGraphics.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
    pGraphics.setRenderingHint(RenderingHints.KEY_RENDERING, RenderingHints.VALUE_RENDER_SPEED);

    //4. draw the background
    pGraphics.drawImage(m_pEmptyAxisImage, 0, 0, null);

    //5. draw the data points
    Color[] pDataPointColors = m_pColoring.getRenderColors(m_pData);
    int[] pRenderPositions = m_pColoring.getRenderPositions(m_pData);

    if (pRenderPositions == null) {

      for (int i = 0; i < iEntryNum; i++) {
        pGraphics.setColor(pDataPointColors[i]);

        float fXPos = (pXValues[i] - m_fXMin)/m_fXSpan;
        fXPos = (fXPos * m_iDrawingWidth) + ARROW_SPACE;

        float fYPos = (pYValues[i] - m_fYMin)/m_fYSpan;
        fYPos = m_iHeight - ARROW_SPACE - (fYPos * m_iDrawingHeight);

        pGraphics.fillOval((int)fXPos-2, (int)fYPos-2, 4, 4);

      }
    }
    else {

      for (int i = 0; i < iEntryNum; i++) {
        int iEntryID = pRenderPositions[i];
        float fXPos = (pXValues[iEntryID] - m_fXMin)/m_fXSpan;
        fXPos = (fXPos * m_iDrawingWidth) + ARROW_SPACE;

        float fYPos = (pYValues[iEntryID] - m_fYMin)/m_fYSpan;
        fYPos = m_iHeight - ARROW_SPACE - (fYPos * m_iDrawingHeight);

        pGraphics.setColor(pDataPointColors[i]);
        pGraphics.fillOval((int)fXPos-2, (int)fYPos-2, 4, 4);

      }

    }

    //6. set the image
    m_pDataImage = pImage;
    m_bRedrawDataDone = true;

  }


  //draws the selections
  private void redrawSelections(Graphics2D pGraphics) {

    //1. set the basic settings for the selections
    pGraphics.setColor(Color.GREEN);
    pGraphics.setStroke(new BasicStroke(2.0f));

    //2. process the selections
    Vector<Selection> pSelections = m_pSelectionList.getLocalSelections(VIEW_ID_SCATTERPLOT);
    int iSelectionNum = pSelections.size();
    
    for (int i = 0; i < iSelectionNum; i++) {

      //2.1. get the selection dimension
      ScatterplotSelection pCurrSelection = (ScatterplotSelection)pSelections.get(i);
      int iSelectionXDim = pCurrSelection.m_iXdimID;
      int iSelectionYDim = pCurrSelection.m_iYdimID;

      //2.2. calculate with which axes the selectionDimensions correspond
      int iXCorrespondent, iYCorrespondent;

      if (iSelectionXDim == m_iXDimID) {
        if (iSelectionXDim == m_iYDimID) {
          iXCorrespondent = CORR_BOTH;
        }
        else {
          iXCorrespondent = CORR_X;
        }
      }
      else if (iSelectionXDim == m_iYDimID) {
        iXCorrespondent = CORR_Y;
      }
      else {
        iXCorrespondent = CORR_NO;
      }

      if (iSelectionYDim == m_iXDimID) {
        if (iSelectionYDim == m_iYDimID) {
          iYCorrespondent = CORR_BOTH;
        }
        else {
          iYCorrespondent = CORR_X;
        }      }
      else if (iSelectionYDim == m_iYDimID) {
        iYCorrespondent = CORR_Y;
      }
      else {
        iYCorrespondent = CORR_NO;
      }

      //System.out.println(iXCorrespondent + ", " + iYCorrespondent);

      //2.3. skip if no correspondence
      if ((iXCorrespondent == CORR_NO) && (iYCorrespondent == CORR_NO)) continue;

      //2.4. if x axis corresponds to both scatterplot axes
      if (iXCorrespondent == CORR_BOTH) {
        int iXMin, iXMax, iYMin, iYMax;

        iXMin = getXPixelPosition(pCurrSelection.getLowerXBorder());
        iXMax = getXPixelPosition(pCurrSelection.getUpperXBorder());

        iYMin = getYPixelPosition(pCurrSelection.getLowerXBorder());
        iYMax = getYPixelPosition(pCurrSelection.getUpperXBorder());

        //make sure that iXMin is smaller than iXMax
        if (iXMin > iXMax) {
          int iSwap = iXMin;
          iXMin = iXMax;
          iXMax = iSwap;
        }

        //make sure that iYMin is smaller than iXMax
        if (iYMin > iYMax) {
          int iSwap = iYMin;
          iYMin = iYMax;
          iYMax = iSwap;
        }


        pGraphics.drawRect(iXMin, iYMin, iXMax-iXMin, iYMax-iYMin);
        //System.out.println("hallo1");

        continue;
      }

      //if y axis corresponds to both scatterplot axes
      if (iYCorrespondent == CORR_BOTH) {
        int iXMin, iXMax, iYMin, iYMax;

        iXMin = getXPixelPosition(pCurrSelection.getLowerYBorder());
        iXMax = getXPixelPosition(pCurrSelection.getUpperYBorder());

        iYMin = getYPixelPosition(pCurrSelection.getLowerYBorder());
        iYMax = getYPixelPosition(pCurrSelection.getUpperYBorder());


        //make sure that iXMin is smaller than iXMax
        if (iXMin > iXMax) {
          int iSwap = iXMin;
          iXMin = iXMax;
          iXMax = iSwap;
        }

        //make sure that iXMin is smaller than iXMax
        if (iYMin > iYMax) {
          int iSwap = iYMin;
          iYMin = iYMax;
          iYMax = iSwap;
        }

        pGraphics.drawRect(iXMin, iYMin, iXMax-iXMin, iYMax-iYMin);

        continue;
      }

      //calculate iXMin, iXMax, iYMin, iYMax
      int iXMin, iXMax, iYMin, iYMax;

      if (iXCorrespondent == CORR_X)  {
        iXMin = getXPixelPosition(pCurrSelection.getLowerXBorder());
        iXMax = getXPixelPosition(pCurrSelection.getUpperXBorder());
      }
      else if (iYCorrespondent == CORR_X) {
        iXMin = getXPixelPosition(pCurrSelection.getLowerYBorder());
        iXMax = getXPixelPosition(pCurrSelection.getUpperYBorder());
      }
      else {
        iXMin = ARROW_SPACE;
        iXMax = m_iDrawingWidth + ARROW_SPACE;
      }

      if (iYCorrespondent == CORR_Y) {
        iYMin = getYPixelPosition(pCurrSelection.getLowerYBorder());
        iYMax = getYPixelPosition(pCurrSelection.getUpperYBorder());
      }
      else if (iXCorrespondent == CORR_Y) {
        iYMin = getYPixelPosition(pCurrSelection.getLowerXBorder());
        iYMax = getYPixelPosition(pCurrSelection.getUpperXBorder());
      }
      else {
        iYMin = ARROW_TOP_LENGTH;
        iYMax = m_iDrawingHeight + ARROW_TOP_LENGTH;
      }




      //make sure that iXMin is smaller than iXMax
      if (iXMin > iXMax) {
        int iSwap = iXMin;
        iXMin = iXMax;
        iXMax = iSwap;
      }


      //make sure that iXMin is smaller than iXMax
      if (iYMin > iYMax) {
        int iSwap = iYMin;
        iYMin = iYMax;
        iYMax = iSwap;
      }
      //draw selection
      pGraphics.drawRect(iXMin, iYMin, iXMax-iXMin, iYMax-iYMin);

    }



  }




  ////////////////////////////////////////
  //Coordinate calculations             //
  ////////////////////////////////////////

  // calculates the pixel position for a certain value on the x axes
  public int getXPixelPosition(float fValue) {
    float fXPos = (fValue - m_fXMin)/m_fXSpan;
    fXPos = (fXPos * m_iDrawingWidth) + ARROW_SPACE;

    return (int)fXPos;
  }

  // calculates the pixel position for a certain value on the y axes
  public int getYPixelPosition(float fValue) {
    float fYPos = (fValue - m_fYMin)/m_fYSpan;
    fYPos = m_iHeight - ARROW_SPACE - (fYPos * m_iDrawingHeight);

    return (int)fYPos;
  }




  // calculates the float value for a given pixel position on the x axes
  public float getCorrespondingXValue(int iXPos) {
    float fXValue = (float)(iXPos - ARROW_SPACE)/(float)m_iDrawingWidth;
    fXValue = fXValue*m_fXSpan + m_fXMin;

    return fXValue;

  }

  // calculates the float value for a given pixel position on the y axes
  public float getCorrespondingYValue(int iYPos) {

    float fYValue = -(float)(iYPos - m_iHeight + ARROW_SPACE)/m_iDrawingHeight;
    fYValue = fYValue*m_fYSpan + m_fYMin;

    return fYValue;
  }

  public float calculateCorrespondingXDistanceValue(int iPixelDistance) {
    float fNormVal = (float)iPixelDistance / (float)m_iDrawingWidth;
    return fNormVal * m_fXSpan;
  }
  public float calculateCorrespondingYDistanceValue(int iPixelDistance) {
    float fNormVal = - (float)iPixelDistance / (float)m_iDrawingHeight;
    return fNormVal * m_fYSpan;
  }





  ////////////////////////////////////////
  //MouseListener methods               //
  ////////////////////////////////////////
  //mouse clicked: show selection menu
  public void mouseClicked(MouseEvent e) {
    m_pInteraction.mouseClicked(e);
  }

  public void mouseEntered(MouseEvent e) {
    m_pInteraction.mouseEntered(e);
  }

  public void mouseExited(MouseEvent e) {
    m_pInteraction.mouseExited(e);
  }

  public void mousePressed(MouseEvent e) {
    m_pInteraction.mousePressed(e);
  }

  public void mouseReleased(MouseEvent e) {
    m_pInteraction.mouseReleased(e);
  }


  ////////////////////////////////////////
  //MouseMotionListener methods         //
  ////////////////////////////////////////
  public void mouseDragged(MouseEvent e) {
    m_pInteraction.mouseDragged(e);
  }

  public void mouseMoved(MouseEvent e) {
    m_pInteraction.mouseMoved(e);
  }





  //////////////////////////////////////////
  // interface for the key events         //
  // triggered from the mainframe         //
  //////////////////////////////////////////

  public void setControlPressed(boolean pressed) {
    m_pInteraction.setControlPressed(pressed);
  }



  //////////////////////////////////////////
  // GET METHODS                          //
  //////////////////////////////////////////
  public float[] getDOI() {
    return m_pSelectionList.getLocalDOI(VIEW_ID_PAARCOORD);
  }



  public void renderImage(BufferedImage pImage) {
    //1. make a simple check
    if (pImage == null) return;
    
    //2. save old size
    int iOldWidth = getWidth();
    int iOldHeight = getHeight();

    //3. paint on the image
    Graphics2D pGraphics = pImage.createGraphics();
    int iWidth = pImage.getWidth();
    int iHeight = pImage.getHeight();
    setSize(iWidth, iHeight);
    m_bRedrawAll = true;
    paint(pGraphics);

    //4. reset the size
    setSize(iOldWidth, iOldHeight);


  }


  public int getXDimID() {
    return m_iXDimID;
  }

  public int getYDimID() {
    return m_iYDimID;
  }

  public int getDrawingWidth() {
    return m_iDrawingWidth;
  }

  public int getDrawingHeight() {
    return m_iDrawingHeight;
  }


  //////////////////////////////////////////
  // SET METHODS                          //
  //////////////////////////////////////////


  /* sets the x axis */
  public void setXAxis(int iAxisID) {
    m_iXDimID = iAxisID;

    m_fXMin = m_pData.getDimensionMinimum(m_iXDimID);
    m_fXMax = m_pData.getDimensionMaximum(m_iXDimID);
    m_fXSpan = m_fXMax - m_fXMin;

    m_pXDimName = m_pData.getDimensionName(m_iXDimID);

    repaintAll();

  }

  /* sets the y axis */
  public void setYAxis(int iAxisID) {
    m_iYDimID = iAxisID;

    m_fYMin = m_pData.getDimensionMinimum(m_iYDimID);
    m_fYMax = m_pData.getDimensionMaximum(m_iYDimID);
    m_fYSpan = m_fYMax - m_fYMin;

    m_pYDimName = m_pData.getDimensionName(m_iYDimID);


    repaintAll();

  }



  //sets a new dataset
  public void setData(Data pData) {
    //1. make a check
    if (pData == null) return;
    
    //2. set the data
    m_pData = pData;

    setXAxis(0);
    setYAxis(1);

    //3. do some painting
    m_bRedrawAll = true;
    repaint();
  }


  public void setInteraction(Interaction pInteraction) {
    //1. make a check
    if (pInteraction == null) return;
    
    m_pInteraction = pInteraction;
  }



  //////////////////////////////////////////
  // CALLS FOR REPAINTING                 //
  //////////////////////////////////////////
  public void repaintSelections() {
    repaint();
  }

  public void repaintData() {
    m_bRedrawData = true;
    repaint();
  }


  //initializes a redraw of the view. redrawBackground indicates if all data items should be redrawn too.
  public void repaintAll() {
    m_bRedrawAll = true;
    repaint();
  }


  //////////////////////////////////////////
  // ICOLORINGLISTENER METHODS            //
  //////////////////////////////////////////

  /** Invoked when a change in the coloring of the data items occurs.
   *
   *  @param coloring IViewColoring object managing the colors of the data items.
   */
  public void coloringChanged(IViewColoring pColoring) {
    //1. make a simple check
    if (pColoring == null) return;
    
    //2. set the coloring and repaint the data
    m_pColoring = pColoring;
    repaintData();
  }


  //////////////////////////////////////////
  // RUNNABLE INTERFACE                   //
  //////////////////////////////////////////
  public void run() {
    redrawDataImage();

    Graphics2D pGraphics = (Graphics2D)getGraphics();

    pGraphics.drawImage(m_pDataImage, 0, 0, this);

    //draw selections
    redrawSelections(pGraphics);

  }


}